@@ -50,6 +50,8 @@ enum aws_auth_errors {
     AWS_AUTH_SSO_TOKEN_EXPIRED,
     AWS_AUTH_CREDENTIALS_PROVIDER_SSO_SOURCE_FAILURE,
     AWS_AUTH_IMDS_CLIENT_SOURCE_FAILURE,
+    AWS_AUTH_PROFILE_STS_CREDENTIALS_PROVIDER_CYCLE_FAILURE,
+    AWS_AUTH_CREDENTIALS_PROVIDER_ECS_INVALID_TOKEN_FILE_PATH,
 
     AWS_AUTH_ERROR_END_RANGE = AWS_ERROR_ENUM_END_RANGE(AWS_C_AUTH_PACKAGE_ID)
 };
