@@ -16,6 +16,7 @@ struct aws_http_connection_manager;
 struct aws_http_make_request_options;
 struct aws_http_stream;
 struct aws_json_value;
+struct aws_profile;
 
 /*
  * Internal struct tracking an asynchronous credentials query.
@@ -31,7 +32,7 @@ struct aws_credentials_query {
 
 typedef struct aws_http_connection_manager *(aws_http_connection_manager_new_fn)(
     struct aws_allocator *allocator,
-    struct aws_http_connection_manager_options *options);
+    const struct aws_http_connection_manager_options *options);
 typedef void(aws_http_connection_manager_release_fn)(struct aws_http_connection_manager *manager);
 typedef void(aws_http_connection_manager_acquire_connection_fn)(
     struct aws_http_connection_manager *manager,
@@ -68,11 +69,14 @@ struct aws_auth_http_system_vtable {
     aws_http_stream_release_fn *aws_http_stream_release;
 
     aws_http_connection_close_fn *aws_http_connection_close;
+
+    int (*aws_high_res_clock_get_ticks)(uint64_t *timestamp);
 };
 
 enum aws_parse_credentials_expiration_format {
     AWS_PCEF_STRING_ISO_8601_DATE,
     AWS_PCEF_NUMBER_UNIX_EPOCH,
+    AWS_PCEF_NUMBER_UNIX_EPOCH_MS,
 };
 
 struct aws_parse_credentials_from_json_doc_options {
@@ -80,6 +84,7 @@ struct aws_parse_credentials_from_json_doc_options {
     const char *secret_access_key_name;
     const char *token_name;
     const char *expiration_name;
+    const char *top_level_object_name;
     enum aws_parse_credentials_expiration_format expiration_format;
     bool token_required;
     bool expiration_required;
@@ -154,12 +159,39 @@ struct aws_credentials *aws_parse_credentials_from_aws_json_object(
 AWS_AUTH_API
 struct aws_credentials *aws_parse_credentials_from_json_document(
     struct aws_allocator *allocator,
-    const char *json_document,
+    struct aws_byte_cursor json_document,
     const struct aws_parse_credentials_from_json_doc_options *options);
 
 AWS_AUTH_API
 enum aws_retry_error_type aws_credentials_provider_compute_retry_error_type(int response_code, int error_code);
 
+/*
+ * Constructs an endpoint in the format of service_name.region.amazonaws.com
+ * If the region is cn-north-1 or cn-northwest-1, .cn is appended to support China-specific regional endpoints.
+ */
+AWS_AUTH_API
+int aws_credentials_provider_construct_regional_endpoint(
+    struct aws_allocator *allocator,
+    struct aws_string **out_endpoint,
+    const struct aws_string *region,
+    const struct aws_string *service_name);
+
+/*
+ * Loads an aws config profile collection
+ */
+AWS_AUTH_API
+struct aws_profile_collection *aws_load_profile_collection_from_config_file(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor config_file_name_override);
+
+/*
+ * Resolve region from environment in the following order:
+ * 1. AWS_REGION
+ * 2. AWS_DEFAULT_REGION
+ */
+AWS_AUTH_API
+struct aws_string *aws_credentials_provider_resolve_region_from_env(struct aws_allocator *allocator);
+
 AWS_EXTERN_C_END
 
 #endif /* AWS_AUTH_CREDENTIALS_PRIVATE_H */
