@@ -14,8 +14,21 @@ struct openssl_aes_cipher {
     struct aws_byte_buf working_buffer;
 };
 
+static struct aws_byte_cursor s_empty_plain_text = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("");
+
 static int s_encrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor input, struct aws_byte_buf *out) {
 
+    /*
+     * Openssl 1.1.1 and its derivatives like aws-lc and boringssl do not handle
+     * the case of null input of 0 len gracefully in update (it succeeds, but
+     * finalize after it will fail). Openssl 3.0 fixed this. Other crypto implementations
+     * do not have similar issue.
+     * To workaround the issue, replace null cursor with empty cursor.
+     */
+    if (input.len == 0) {
+        input = s_empty_plain_text;
+    }
+
     size_t required_buffer_space = input.len + cipher->block_size;
 
     if (aws_symmetric_cipher_try_ensure_sufficient_buffer_space(out, required_buffer_space)) {
@@ -28,7 +41,7 @@ static int s_encrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor
     int len_written = (int)(available_write_space);
     if (!EVP_EncryptUpdate(
             openssl_cipher->encryptor_ctx, out->buffer + out->len, &len_written, input.ptr, (int)input.len)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -47,7 +60,7 @@ static int s_finalize_encryption(struct aws_symmetric_cipher *cipher, struct aws
 
     int len_written = (int)(out->capacity - out->len);
     if (!EVP_EncryptFinal_ex(openssl_cipher->encryptor_ctx, out->buffer + out->len, &len_written)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -69,8 +82,7 @@ static int s_decrypt(struct aws_symmetric_cipher *cipher, struct aws_byte_cursor
     int len_written = (int)available_write_space;
     if (!EVP_DecryptUpdate(
             openssl_cipher->decryptor_ctx, out->buffer + out->len, &len_written, input.ptr, (int)input.len)) {
-        cipher->good = false;
-
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -89,7 +101,7 @@ static int s_finalize_decryption(struct aws_symmetric_cipher *cipher, struct aws
 
     int len_written = (int)out->capacity - out->len;
     if (!EVP_DecryptFinal_ex(openssl_cipher->decryptor_ctx, out->buffer + out->len, &len_written)) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -130,7 +142,7 @@ static int s_clear_reusable_state(struct aws_symmetric_cipher *cipher) {
     EVP_CIPHER_CTX_cleanup(openssl_cipher->encryptor_ctx);
     EVP_CIPHER_CTX_cleanup(openssl_cipher->decryptor_ctx);
     aws_byte_buf_secure_zero(&openssl_cipher->working_buffer);
-    cipher->good = true;
+    cipher->state = AWS_SYMMETRIC_CIPHER_READY;
     return AWS_OP_SUCCESS;
 }
 
@@ -215,7 +227,7 @@ struct aws_symmetric_cipher *aws_aes_cbc_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -306,7 +318,7 @@ struct aws_symmetric_cipher *aws_aes_ctr_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -326,7 +338,7 @@ static int s_finalize_gcm_encryption(struct aws_symmetric_cipher *cipher, struct
                     EVP_CTRL_GCM_GET_TAG,
                     (int)cipher->tag.capacity,
                     cipher->tag.buffer)) {
-                cipher->good = false;
+                cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
                 return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             }
             cipher->tag.len = AWS_AES_256_CIPHER_BLOCK_SIZE;
@@ -466,7 +478,7 @@ struct aws_symmetric_cipher *aws_aes_gcm_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
@@ -492,7 +504,7 @@ static int s_key_wrap_finalize_encryption(struct aws_symmetric_cipher *cipher, s
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
     if (openssl_cipher->working_buffer.len < MIN_CEK_LENGTH_BYTES) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
 
@@ -535,7 +547,7 @@ static int s_key_wrap_finalize_encryption(struct aws_symmetric_cipher *cipher, s
             /* encrypt the concatenated A and R[I] and store it in B */
             if (!EVP_EncryptUpdate(
                     openssl_cipher->encryptor_ctx, b.buffer, &b_out_len, temp_input.buffer, (int)temp_input.capacity)) {
-                cipher->good = false;
+                cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
                 return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             }
 
@@ -560,7 +572,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
     struct openssl_aes_cipher *openssl_cipher = cipher->impl;
 
     if (openssl_cipher->working_buffer.len < MIN_CEK_LENGTH_BYTES + KEYWRAP_BLOCK_SIZE) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_STATE);
     }
 
@@ -607,7 +619,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
             /* Decrypt the concatenated buffer */
             if (!EVP_DecryptUpdate(
                     openssl_cipher->decryptor_ctx, b.buffer, &b_out_len, temp_input.buffer, (int)temp_input.capacity)) {
-                cipher->good = false;
+                cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
                 return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
             }
 
@@ -625,7 +637,7 @@ static int s_key_wrap_finalize_decryption(struct aws_symmetric_cipher *cipher, s
     /* here we perform the integrity check to make sure A == 0xA6A6A6A6A6A6A6A6 */
     for (size_t i = 0; i < KEYWRAP_BLOCK_SIZE; ++i) {
         if (a[i] != INTEGRITY_VALUE) {
-            cipher->good = false;
+            cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
             return aws_raise_error(AWS_ERROR_CAL_SIGNATURE_VALIDATION_FAILED);
         }
     }
@@ -641,7 +653,7 @@ static int s_init_keywrap_cipher_materials(struct aws_symmetric_cipher *cipher)
           EVP_CIPHER_CTX_set_padding(openssl_cipher->encryptor_ctx, 0)) ||
         !(EVP_DecryptInit_ex(openssl_cipher->decryptor_ctx, EVP_aes_256_ecb(), NULL, cipher->key.buffer, NULL) &&
           EVP_CIPHER_CTX_set_padding(openssl_cipher->decryptor_ctx, 0))) {
-        cipher->good = false;
+        cipher->state = AWS_SYMMETRIC_CIPHER_ERROR;
         return aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
     }
 
@@ -701,7 +713,7 @@ struct aws_symmetric_cipher *aws_aes_keywrap_256_new_impl(
         goto error;
     }
 
-    cipher->cipher_base.good = true;
+    cipher->cipher_base.state = AWS_SYMMETRIC_CIPHER_READY;
     return &cipher->cipher_base;
 
 error:
