@@ -44,6 +44,9 @@ static struct aws_error_info s_errors[] = {
     AWS_DEFINE_ERROR_INFO_S3(AWS_ERROR_S3_EXCEEDS_MEMORY_LIMIT, "Request was not created due to used memory exceeding memory limit."),
     AWS_DEFINE_ERROR_INFO_S3(AWS_ERROR_S3_INVALID_MEMORY_LIMIT_CONFIG, "Specified memory configuration is invalid for the system. "
         "Memory limit should be at least 1GiB. Part size and max part size should be smaller than memory limit."),
+    AWS_DEFINE_ERROR_INFO_S3(AWS_ERROR_S3EXPRESS_CREATE_SESSION_FAILED, "CreateSession call failed when signing with S3 Express."),
+    AWS_DEFINE_ERROR_INFO_S3(AWS_ERROR_S3_INTERNAL_PART_SIZE_MISMATCH_RETRYING_WITH_RANGE, "part_size mismatch, possibly due to wrong object_size_hint. Retrying with Range instead of partNumber."),
+    AWS_DEFINE_ERROR_INFO_S3(AWS_ERROR_S3_REQUEST_HAS_COMPLETED, "Request has already completed, action cannot be performed."),
 };
 /* clang-format on */
 
@@ -101,6 +104,10 @@ const struct aws_s3_platform_info *aws_s3_get_current_platform_info(void) {
     return aws_s3_get_platform_info_for_current_environment(s_loader);
 }
 
+struct aws_byte_cursor aws_s3_get_current_platform_ec2_intance_type(bool cached_only) {
+    return aws_s3_get_ec2_instance_type(s_loader, cached_only);
+}
+
 struct aws_array_list aws_s3_get_platforms_with_recommended_config(void) {
     return aws_s3_get_recommended_platforms(s_loader);
 }
