@@ -13,24 +13,30 @@
 #include <aws/testing/aws_test_harness.h>
 
 enum s3_update_cancel_type {
+    S3_UPDATE_CANCEL_TYPE_NO_CANCEL,
+
     S3_UPDATE_CANCEL_TYPE_MPU_CREATE_NOT_SENT,
     S3_UPDATE_CANCEL_TYPE_MPU_CREATE_COMPLETED,
     S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED,
     S3_UPDATE_CANCEL_TYPE_MPU_ALL_PARTS_COMPLETED,
+    S3_UPDATE_CANCEL_TYPE_MPU_ONGOING_HTTP_REQUESTS,
     S3_UPDATE_CANCEL_TYPE_NUM_MPU_CANCEL_TYPES,
 
     S3_UPDATE_CANCEL_TYPE_MPD_NOTHING_SENT,
     S3_UPDATE_CANCEL_TYPE_MPD_HEAD_OBJECT_SENT,
     S3_UPDATE_CANCEL_TYPE_MPD_HEAD_OBJECT_COMPLETED,
-    S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_SENT,
-    S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_COMPLETED,
+    S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_SENT,
+    S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_COMPLETED,
     S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_SENT,
     S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_COMPLETED,
     S3_UPDATE_CANCEL_TYPE_MPD_TWO_PARTS_COMPLETED,
+    S3_UPDATE_CANCEL_TYPE_MPD_PENDING_STREAMING,
 };
 
 struct s3_cancel_test_user_data {
     enum s3_update_cancel_type type;
+    bool pause;
+    struct aws_s3_meta_request_resume_token *resume_token;
     bool abort_successful;
 };
 
@@ -48,25 +54,32 @@ static bool s_s3_meta_request_update_cancel_test(
     struct aws_s3_auto_ranged_put *auto_ranged_put = meta_request->impl;
     struct aws_s3_auto_ranged_get *auto_ranged_get = meta_request->impl;
 
-    bool call_cancel = false;
+    bool call_cancel_or_pause = false;
     bool block_update = false;
 
     aws_s3_meta_request_lock_synced_data(meta_request);
 
     switch (cancel_test_user_data->type) {
+        case S3_UPDATE_CANCEL_TYPE_NO_CANCEL:
+            break;
+
         case S3_UPDATE_CANCEL_TYPE_MPU_CREATE_NOT_SENT:
-            call_cancel = auto_ranged_put->synced_data.create_multipart_upload_sent != 0;
+            call_cancel_or_pause = auto_ranged_put->synced_data.create_multipart_upload_sent != 0;
             break;
         case S3_UPDATE_CANCEL_TYPE_MPU_CREATE_COMPLETED:
-            call_cancel = auto_ranged_put->synced_data.create_multipart_upload_completed != 0;
+            call_cancel_or_pause = auto_ranged_put->synced_data.create_multipart_upload_completed != 0;
             break;
         case S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED:
-            call_cancel = auto_ranged_put->synced_data.num_parts_completed == 1;
-            block_update = !call_cancel && auto_ranged_put->synced_data.num_parts_started == 1;
+            call_cancel_or_pause = auto_ranged_put->synced_data.num_parts_completed == 1;
+            block_update = !call_cancel_or_pause && auto_ranged_put->synced_data.num_parts_started == 1;
             break;
         case S3_UPDATE_CANCEL_TYPE_MPU_ALL_PARTS_COMPLETED:
-            call_cancel = auto_ranged_put->synced_data.num_parts_completed ==
-                          auto_ranged_put->total_num_parts_from_content_length;
+            call_cancel_or_pause = auto_ranged_put->synced_data.num_parts_completed ==
+                                   auto_ranged_put->total_num_parts_from_content_length;
+            break;
+
+        case S3_UPDATE_CANCEL_TYPE_MPU_ONGOING_HTTP_REQUESTS:
+            call_cancel_or_pause = !aws_linked_list_empty(&meta_request->synced_data.cancellable_http_streams_list);
             break;
 
         case S3_UPDATE_CANCEL_TYPE_NUM_MPU_CANCEL_TYPES:
@@ -74,48 +87,57 @@ static bool s_s3_meta_request_update_cancel_test(
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_NOTHING_SENT:
-            call_cancel = auto_ranged_get->synced_data.num_parts_requested == 0;
+            call_cancel_or_pause = auto_ranged_get->synced_data.num_parts_requested == 0;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_HEAD_OBJECT_SENT:
-            call_cancel = auto_ranged_get->synced_data.head_object_sent != 0;
+            call_cancel_or_pause = auto_ranged_get->synced_data.head_object_sent != 0;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_HEAD_OBJECT_COMPLETED:
-            call_cancel = auto_ranged_get->synced_data.head_object_completed != 0;
+            call_cancel_or_pause = auto_ranged_get->synced_data.head_object_completed != 0;
             break;
 
-        case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_SENT:
-            call_cancel = auto_ranged_get->synced_data.get_without_range_sent != 0;
+        case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_SENT:
+            call_cancel_or_pause = auto_ranged_get->synced_data.object_range_known != 0 &&
+                                   auto_ranged_get->synced_data.num_parts_requested > 0;
             break;
 
-        case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_COMPLETED:
-            call_cancel = auto_ranged_get->synced_data.get_without_range_completed != 0;
+        case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_COMPLETED:
+            call_cancel_or_pause = auto_ranged_get->synced_data.num_parts_completed > 0;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_SENT:
-            call_cancel = auto_ranged_get->synced_data.num_parts_requested == 1;
+            call_cancel_or_pause = auto_ranged_get->synced_data.num_parts_requested == 1;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_ONE_PART_COMPLETED:
-            call_cancel = auto_ranged_get->synced_data.num_parts_completed == 1;
+            call_cancel_or_pause = auto_ranged_get->synced_data.num_parts_completed == 1;
 
             /* Prevent other parts from being queued while we wait for this one to complete. */
-            block_update = !call_cancel && auto_ranged_get->synced_data.num_parts_requested == 1;
+            block_update = !call_cancel_or_pause && auto_ranged_get->synced_data.num_parts_requested == 1;
             break;
 
         case S3_UPDATE_CANCEL_TYPE_MPD_TWO_PARTS_COMPLETED:
-            call_cancel = auto_ranged_get->synced_data.num_parts_completed == 2;
+            call_cancel_or_pause = auto_ranged_get->synced_data.num_parts_completed == 2;
 
             /* Prevent other parts from being queued while we wait for these two to complete. */
-            block_update = !call_cancel && auto_ranged_get->synced_data.num_parts_requested == 2;
+            block_update = !call_cancel_or_pause && auto_ranged_get->synced_data.num_parts_requested == 2;
+            break;
+
+        case S3_UPDATE_CANCEL_TYPE_MPD_PENDING_STREAMING:
+            call_cancel_or_pause =
+                aws_priority_queue_size(&meta_request->synced_data.pending_body_streaming_requests) > 0;
             break;
     }
 
     aws_s3_meta_request_unlock_synced_data(meta_request);
-
-    if (call_cancel) {
-        aws_s3_meta_request_cancel(meta_request);
+    if (call_cancel_or_pause) {
+        if (cancel_test_user_data->pause) {
+            aws_s3_meta_request_pause(meta_request, &cancel_test_user_data->resume_token);
+        } else {
+            aws_s3_meta_request_cancel(meta_request);
+        }
     }
 
     if (block_update) {
@@ -175,7 +197,8 @@ static struct aws_s3_meta_request *s_meta_request_factory_patch_update_cancel_te
 static int s3_cancel_test_helper_ex(
     struct aws_allocator *allocator,
     enum s3_update_cancel_type cancel_type,
-    bool async_input_stream) {
+    bool async_input_stream,
+    bool pause) {
 
     AWS_ASSERT(allocator);
 
@@ -184,6 +207,7 @@ static int s3_cancel_test_helper_ex(
 
     struct s3_cancel_test_user_data test_user_data = {
         .type = cancel_type,
+        .pause = pause,
     };
 
     tester.user_data = &test_user_data;
@@ -221,13 +245,49 @@ static int s3_cancel_test_helper_ex(
         };
 
         ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &options, &meta_request_test_results));
-        ASSERT_INT_EQUALS(AWS_ERROR_S3_CANCELED, meta_request_test_results.finished_error_code);
+        int expected_error_code = pause ? AWS_ERROR_S3_PAUSED : AWS_ERROR_S3_CANCELED;
+        ASSERT_INT_EQUALS(expected_error_code, meta_request_test_results.finished_error_code);
+
+        if (cancel_type == S3_UPDATE_CANCEL_TYPE_MPU_ONGOING_HTTP_REQUESTS) {
+            /* Check the metric and see we have at least a request completed with AWS_ERROR_S3_CANCELED */
+            /* The meta request completed, we can access the synced data now. */
+            struct aws_array_list *metrics_list = &meta_request_test_results.synced_data.metrics;
+            bool cancelled_successfully = false;
+            for (size_t i = 0; i < aws_array_list_length(metrics_list); ++i) {
+                struct aws_s3_request_metrics *metrics = NULL;
+                aws_array_list_get_at(metrics_list, (void **)&metrics, i);
+                if (metrics->crt_info_metrics.error_code == expected_error_code) {
+                    cancelled_successfully = true;
+                    break;
+                }
+            }
+            ASSERT_TRUE(cancelled_successfully);
+        }
 
         aws_s3_meta_request_test_results_clean_up(&meta_request_test_results);
-
-        if (cancel_type != S3_UPDATE_CANCEL_TYPE_MPU_CREATE_NOT_SENT) {
+        if (cancel_type != S3_UPDATE_CANCEL_TYPE_MPU_CREATE_NOT_SENT && !pause) {
             ASSERT_TRUE(test_user_data.abort_successful);
         }
+        if (pause) {
+            /* Resume the paused request. */
+            ASSERT_NOT_NULL(test_user_data.resume_token);
+            test_user_data.type = S3_UPDATE_CANCEL_TYPE_NO_CANCEL;
+            struct aws_s3_tester_meta_request_options resume_options = {
+                .allocator = allocator,
+                .client = client,
+                .meta_request_type = AWS_S3_META_REQUEST_TYPE_PUT_OBJECT,
+                .validate_type = AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_SUCCESS,
+                .put_options =
+                    {
+                        .ensure_multipart = true,
+                        .async_input_stream = async_input_stream,
+                        .resume_token = test_user_data.resume_token,
+                    },
+            };
+
+            ASSERT_SUCCESS(aws_s3_tester_send_meta_request_with_options(&tester, &resume_options, NULL));
+            aws_s3_meta_request_resume_token_release(test_user_data.resume_token);
+        }
 
         /* TODO: perform additional verification with list-multipart-uploads */
 
@@ -236,8 +296,8 @@ static int s3_cancel_test_helper_ex(
         struct aws_s3_meta_request_test_results meta_request_test_results;
         aws_s3_meta_request_test_results_init(&meta_request_test_results, allocator);
 
-        // Range for the second 16k
-        const struct aws_byte_cursor range = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("bytes=16384-32767");
+        /* Specify a range without start-range to trigger HeadRequest */
+        const struct aws_byte_cursor range = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("bytes=-32767");
 
         struct aws_s3_tester_meta_request_options options = {
             .allocator = allocator,
@@ -246,7 +306,9 @@ static int s3_cancel_test_helper_ex(
             .validate_type = AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_FAILURE,
             .get_options =
                 {
-                    .object_path = g_pre_existing_object_1MB,
+                    /* Note 1: 10MB object with 16KB parts, so that tests have many requests in-flight.
+                     * We want to try and stress stuff like parts arriving out of order. */
+                    .object_path = g_pre_existing_object_10MB,
                 },
         };
 
@@ -259,11 +321,11 @@ static int s3_cancel_test_helper_ex(
                 options.get_options.object_range = range;
                 break;
 
-            case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_SENT:
+            case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_SENT:
                 options.get_options.object_path = g_pre_existing_empty_object;
                 break;
 
-            case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_COMPLETED:
+            case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_COMPLETED:
                 options.get_options.object_path = g_pre_existing_empty_object;
                 break;
 
@@ -284,7 +346,7 @@ static int s3_cancel_test_helper_ex(
 }
 
 static int s3_cancel_test_helper(struct aws_allocator *allocator, enum s3_update_cancel_type cancel_type) {
-    return s3_cancel_test_helper_ex(allocator, cancel_type, false /*async_input_stream*/);
+    return s3_cancel_test_helper_ex(allocator, cancel_type, false /*async_input_stream*/, false /*pause*/);
 }
 
 static int s3_cancel_test_helper_fc(
@@ -377,11 +439,11 @@ static int s3_cancel_test_helper_fc(
                 options.get_options.object_range = range;
                 break;
 
-            case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_SENT:
+            case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_SENT:
                 options.get_options.object_path = g_pre_existing_empty_object;
                 break;
 
-            case S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_COMPLETED:
+            case S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_COMPLETED:
                 options.get_options.object_path = g_pre_existing_empty_object;
                 break;
 
@@ -459,8 +521,8 @@ AWS_TEST_CASE(test_s3_cancel_mpu_one_part_completed_async, s_test_s3_cancel_mpu_
 static int s_test_s3_cancel_mpu_one_part_completed_async(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
-    ASSERT_SUCCESS(
-        s3_cancel_test_helper_ex(allocator, S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED, true /*async_input_stream*/));
+    ASSERT_SUCCESS(s3_cancel_test_helper_ex(
+        allocator, S3_UPDATE_CANCEL_TYPE_MPU_ONE_PART_COMPLETED, true /*async_input_stream*/, false /*pause*/));
 
     return 0;
 }
@@ -474,6 +536,25 @@ static int s_test_s3_cancel_mpu_all_parts_completed(struct aws_allocator *alloca
     return 0;
 }
 
+AWS_TEST_CASE(test_s3_cancel_mpu_cancellable_requests, s_test_s3_cancel_mpu_cancellable_requests)
+static int s_test_s3_cancel_mpu_cancellable_requests(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    ASSERT_SUCCESS(s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPU_ONGOING_HTTP_REQUESTS));
+
+    return 0;
+}
+
+AWS_TEST_CASE(test_s3_pause_mpu_cancellable_requests, s_test_s3_pause_mpu_cancellable_requests)
+static int s_test_s3_pause_mpu_cancellable_requests(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    ASSERT_SUCCESS(s3_cancel_test_helper_ex(
+        allocator, S3_UPDATE_CANCEL_TYPE_MPU_ONGOING_HTTP_REQUESTS, false /*async_input_stream*/, true /*pause*/));
+
+    return 0;
+}
+
 AWS_TEST_CASE(test_s3_cancel_mpd_nothing_sent, s_test_s3_cancel_mpd_nothing_sent)
 static int s_test_s3_cancel_mpd_nothing_sent(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
@@ -528,20 +609,37 @@ static int s_test_s3_cancel_mpd_head_object_completed(struct aws_allocator *allo
     return 0;
 }
 
-AWS_TEST_CASE(test_s3_cancel_mpd_get_without_range_sent, s_test_s3_cancel_mpd_get_without_range_sent)
-static int s_test_s3_cancel_mpd_get_without_range_sent(struct aws_allocator *allocator, void *ctx) {
+AWS_TEST_CASE(
+    test_s3_cancel_mpd_empty_object_get_with_part_number_1_sent,
+    s_test_s3_cancel_mpd_empty_object_get_with_part_number_1_sent)
+static int s_test_s3_cancel_mpd_empty_object_get_with_part_number_1_sent(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
 
-    ASSERT_SUCCESS(s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_SENT));
+    ASSERT_SUCCESS(
+        s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_SENT));
 
     return 0;
 }
 
-AWS_TEST_CASE(test_s3_cancel_mpd_get_without_range_completed, s_test_s3_cancel_mpd_get_without_range_completed)
-static int s_test_s3_cancel_mpd_get_without_range_completed(struct aws_allocator *allocator, void *ctx) {
+AWS_TEST_CASE(
+    test_s3_cancel_mpd_empty_object_get_with_part_number_1_completed,
+    s_test_s3_cancel_mpd_empty_object_get_with_part_number_1_completed)
+static int s_test_s3_cancel_mpd_empty_object_get_with_part_number_1_completed(
+    struct aws_allocator *allocator,
+    void *ctx) {
     (void)ctx;
 
-    ASSERT_SUCCESS(s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPD_GET_WITHOUT_RANGE_COMPLETED));
+    ASSERT_SUCCESS(
+        s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPD_GET_EMPTY_OBJECT_WITH_PART_NUMBER_1_COMPLETED));
+
+    return 0;
+}
+
+AWS_TEST_CASE(test_s3_cancel_mpd_pending_streaming, s_test_s3_cancel_mpd_pending_streaming)
+static int s_test_s3_cancel_mpd_pending_streaming(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    ASSERT_SUCCESS(s3_cancel_test_helper(allocator, S3_UPDATE_CANCEL_TYPE_MPD_PENDING_STREAMING));
 
     return 0;
 }
