@@ -64,6 +64,16 @@ struct aws_byte_cursor g_test_bucket_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERA
 /* If `$CRT_S3_TEST_BUCKET_NAME` envrionment variable is set, use `$CRT_S3_TEST_BUCKET_NAME-public`; otherwise, use
  * aws-c-s3-test-bucket-public */
 struct aws_byte_cursor g_test_public_bucket_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("aws-c-s3-test-bucket-public");
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com */
+struct aws_byte_cursor g_test_s3express_bucket_usw2_az1_endpoint = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
+    "aws-c-s3-test-bucket--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com");
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--us1-az1--x-s3.s3express-use1-az4.us-east-1.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com */
+struct aws_byte_cursor g_test_s3express_bucket_use1_az4_endpoint = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL(
+    "aws-c-s3-test-bucket--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com");
 
 #ifdef BYO_CRYPTO
 /* Under BYO_CRYPTO, this function currently needs to be defined by the user. Defining a null implementation here so
@@ -126,7 +136,7 @@ static int s_s3_test_meta_request_body_callback(
 
     /* If this is an auto-ranged-get meta request, then grab the object range start so that the expected_range_start can
      * be properly offset.*/
-    if (meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT) {
+    if (meta_request->type == AWS_S3_META_REQUEST_TYPE_GET_OBJECT && meta_request->part_size != 0) {
 
         aws_s3_meta_request_lock_synced_data(meta_request);
 
@@ -355,6 +365,28 @@ int aws_s3_tester_init(struct aws_allocator *allocator, struct aws_s3_tester *te
             AWS_BYTE_CURSOR_PRI(g_test_bucket_name));
         tester->public_bucket_name = aws_string_new_from_c_str(allocator, public_bucket_name_buffer);
         g_test_public_bucket_name = aws_byte_cursor_from_string(tester->public_bucket_name);
+
+        char s3express_bucket_usw2_az1_endpoint_buffer[512] = "";
+        snprintf(
+            s3express_bucket_usw2_az1_endpoint_buffer,
+            sizeof(s3express_bucket_usw2_az1_endpoint_buffer),
+            "" PRInSTR "--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com",
+            AWS_BYTE_CURSOR_PRI(g_test_bucket_name));
+        tester->s3express_bucket_usw2_az1_endpoint =
+            aws_string_new_from_c_str(allocator, s3express_bucket_usw2_az1_endpoint_buffer);
+        g_test_s3express_bucket_usw2_az1_endpoint =
+            aws_byte_cursor_from_string(tester->s3express_bucket_usw2_az1_endpoint);
+
+        char s3express_bucket_use1_az4_name_buffer[128] = "";
+        snprintf(
+            s3express_bucket_use1_az4_name_buffer,
+            sizeof(s3express_bucket_use1_az4_name_buffer),
+            "" PRInSTR "--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com",
+            AWS_BYTE_CURSOR_PRI(g_test_bucket_name));
+        tester->s3express_bucket_use1_az4_endpoint =
+            aws_string_new_from_c_str(allocator, s3express_bucket_use1_az4_name_buffer);
+        g_test_s3express_bucket_use1_az4_endpoint =
+            aws_byte_cursor_from_string(tester->s3express_bucket_use1_az4_endpoint);
     }
 
     aws_s3_library_init(allocator);
@@ -698,6 +730,8 @@ void aws_s3_tester_clean_up(struct aws_s3_tester *tester) {
     }
     aws_string_destroy(tester->bucket_name);
     aws_string_destroy(tester->public_bucket_name);
+    aws_string_destroy(tester->s3express_bucket_usw2_az1_endpoint);
+    aws_string_destroy(tester->s3express_bucket_use1_az4_endpoint);
 
     aws_credentials_release(tester->anonymous_creds);
 
@@ -1406,10 +1440,12 @@ int aws_s3_tester_send_meta_request_with_options(
         aws_s3_client_acquire(client);
     }
 
+    bool disable_trailing_checksum =
+        options->checksum_algorithm == AWS_SCA_NONE || options->disable_put_trailing_checksum;
     struct aws_s3_checksum_config checksum_config = {
         .checksum_algorithm = options->checksum_algorithm,
         .validate_response_checksum = options->validate_get_response_checksum,
-        .location = options->checksum_algorithm == AWS_SCA_NONE ? AWS_SCL_NONE : AWS_SCL_TRAILER,
+        .location = disable_trailing_checksum ? AWS_SCL_NONE : AWS_SCL_TRAILER,
         .validate_checksum_algorithms = options->validate_checksum_algorithms,
     };
 
@@ -1419,6 +1455,7 @@ int aws_s3_tester_send_meta_request_with_options(
         .message = options->message,
         .checksum_config = &checksum_config,
         .resume_token = options->put_options.resume_token,
+        .object_size_hint = options->object_size_hint,
     };
 
     if (options->mock_server) {
@@ -1464,6 +1501,12 @@ int aws_s3_tester_send_meta_request_with_options(
 
             struct aws_http_message *message = aws_s3_test_get_object_request_new(
                 allocator, aws_byte_cursor_from_string(host_name), options->get_options.object_path);
+            ASSERT_SUCCESS(aws_s3_message_util_set_multipart_request_path(
+                allocator,
+                NULL /*upload_id*/,
+                options->get_options.part_number,
+                false /*append_uploads_suffix*/,
+                message));
 
             if (options->get_options.object_range.ptr != NULL) {
                 struct aws_http_header range_header = {
