@@ -100,6 +100,8 @@ struct aws_s3_tester {
 
     struct aws_string *bucket_name;
     struct aws_string *public_bucket_name;
+    struct aws_string *s3express_bucket_usw2_az1_endpoint;
+    struct aws_string *s3express_bucket_use1_az4_endpoint;
 
     struct {
         struct aws_mutex lock;
@@ -159,6 +161,7 @@ struct aws_s3_tester_meta_request_options {
     enum aws_s3_checksum_algorithm checksum_algorithm;
     struct aws_array_list *validate_checksum_algorithms;
     enum aws_s3_checksum_algorithm expected_validate_checksum_alg;
+    bool disable_put_trailing_checksum;
 
     /* override client signing config */
     struct aws_signing_config_aws *signing_config;
@@ -181,6 +184,8 @@ struct aws_s3_tester_meta_request_options {
     struct {
         struct aws_byte_cursor object_path;
         struct aws_byte_cursor object_range;
+        /* Get the part from S3, starts from 1. 0 means not set. */
+        int part_number;
     } get_options;
 
     /* Put Object Meta request specific options. */
@@ -213,6 +218,8 @@ struct aws_s3_tester_meta_request_options {
     uint32_t dont_wait_for_shutdown : 1;
 
     uint32_t mrap_test : 1;
+
+    uint64_t *object_size_hint;
 };
 
 /* TODO Rename to something more generic such as "aws_s3_meta_request_test_data" */
@@ -489,5 +496,13 @@ extern struct aws_byte_cursor g_test_bucket_name;
  * aws-c-s3-test-bucket-public
  */
 extern struct aws_byte_cursor g_test_public_bucket_name;
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--usw2-az1--x-s3.s3express-usw2-az1.us-west-2.amazonaws.com */
+extern struct aws_byte_cursor g_test_s3express_bucket_usw2_az1_endpoint;
+/* If `$CRT_S3_TEST_BUCKET_NAME` environment variable is set, use
+ * `$CRT_S3_TEST_BUCKET_NAME--us1-az1--x-s3.s3express-use1-az4.us-east-1.amazonaws.com`; otherwise, use
+ * aws-c-s3-test-bucket--use1-az4--x-s3.s3express-use1-az4.us-east-1.amazonaws.com */
+extern struct aws_byte_cursor g_test_s3express_bucket_use1_az4_endpoint;
 
 #endif /* AWS_S3_TESTER_H */
