@@ -151,17 +151,24 @@ OPENSSL_INLINE int montgomery_use_s2n_bignum(unsigned int num) {
 
 #endif
 
+void bn_mont_ctx_init(BN_MONT_CTX *mont) {
+  OPENSSL_memset(mont, 0, sizeof(BN_MONT_CTX));
+  BN_init(&mont->RR);
+  BN_init(&mont->N);
+}
+
+void bn_mont_ctx_cleanup(BN_MONT_CTX *mont) {
+  BN_free(&mont->RR);
+  BN_free(&mont->N);
+}
+
 BN_MONT_CTX *BN_MONT_CTX_new(void) {
   BN_MONT_CTX *ret = OPENSSL_malloc(sizeof(BN_MONT_CTX));
-
   if (ret == NULL) {
     return NULL;
   }
 
-  OPENSSL_memset(ret, 0, sizeof(BN_MONT_CTX));
-  BN_init(&ret->RR);
-  BN_init(&ret->N);
-
+  bn_mont_ctx_init(ret);
   return ret;
 }
 
@@ -170,8 +177,7 @@ void BN_MONT_CTX_free(BN_MONT_CTX *mont) {
     return;
   }
 
-  BN_free(&mont->RR);
-  BN_free(&mont->N);
+  bn_mont_ctx_cleanup(mont);
   OPENSSL_free(mont);
 }
 
