@@ -86,15 +86,14 @@
 DEFINE_STATIC_EX_DATA_CLASS(g_ec_ex_data_class)
 
 static EC_WRAPPED_SCALAR *ec_wrapped_scalar_new(const EC_GROUP *group) {
-  EC_WRAPPED_SCALAR *wrapped = OPENSSL_malloc(sizeof(EC_WRAPPED_SCALAR));
+  EC_WRAPPED_SCALAR *wrapped = OPENSSL_zalloc(sizeof(EC_WRAPPED_SCALAR));
   if (wrapped == NULL) {
     return NULL;
   }
 
-  OPENSSL_memset(wrapped, 0, sizeof(EC_WRAPPED_SCALAR));
   wrapped->bignum.d = wrapped->scalar.words;
-  wrapped->bignum.width = group->order->N.width;
-  wrapped->bignum.dmax = group->order->N.width;
+  wrapped->bignum.width = group->order.N.width;
+  wrapped->bignum.dmax = group->order.N.width;
   wrapped->bignum.flags = BN_FLG_STATIC_DATA;
   return wrapped;
 }
@@ -106,13 +105,11 @@ static void ec_wrapped_scalar_free(EC_WRAPPED_SCALAR *scalar) {
 EC_KEY *EC_KEY_new(void) { return EC_KEY_new_method(NULL); }
 
 EC_KEY *EC_KEY_new_method(const ENGINE *engine) {
-  EC_KEY *ret = OPENSSL_malloc(sizeof(EC_KEY));
+  EC_KEY *ret = OPENSSL_zalloc(sizeof(EC_KEY));
   if (ret == NULL) {
     return NULL;
   }
 
-  OPENSSL_memset(ret, 0, sizeof(EC_KEY));
-
   if (engine) {
     ret->ecdsa_meth = ENGINE_get_ECDSA_method(engine);
   }
@@ -387,7 +384,7 @@ int EC_KEY_check_fips(const EC_KEY *key) {
   // ec_felem_to_bignum() calls BN_bin2bn() which sets the `neg` flag to 0.
   EC_POINT *pub_key = key->pub_key;
   EC_GROUP *group = key->pub_key->group;
-  if(ec_felem_equal(group, &group->one, &pub_key->raw.Z)) {
+  if(ec_felem_equal(group, ec_felem_one(group), &pub_key->raw.Z)) {
     BIGNUM *x = BN_new();
     BIGNUM *y = BN_new();
     int check_ret = 1;
@@ -399,8 +396,8 @@ int EC_KEY_check_fips(const EC_KEY *key) {
       // Error already written to error queue by |bn_wexpand|.
       check_ret = 0;
     } else if (BN_is_negative(x) || BN_is_negative(y) ||
-               BN_cmp(x, &group->field) >= 0 ||
-               BN_cmp(y, &group->field) >= 0) {
+               BN_cmp(x, &group->field.N) >= 0 ||
+               BN_cmp(y, &group->field.N) >= 0) {
       OPENSSL_PUT_ERROR(EC, EC_R_COORDINATES_OUT_OF_RANGE);
       check_ret = 0;
     }
