@@ -84,12 +84,11 @@ OPENSSL_DECLARE_ERROR_REASON(EVP, EMPTY_PSK)
 EVP_PKEY *EVP_PKEY_new(void) {
   EVP_PKEY *ret;
 
-  ret = OPENSSL_malloc(sizeof(EVP_PKEY));
+  ret = OPENSSL_zalloc(sizeof(EVP_PKEY));
   if (ret == NULL) {
     return NULL;
   }
 
-  OPENSSL_memset(ret, 0, sizeof(EVP_PKEY));
   ret->type = EVP_PKEY_NONE;
   ret->references = 1;
 
@@ -210,6 +209,31 @@ int EVP_PKEY_id(const EVP_PKEY *pkey) {
   return pkey->type;
 }
 
+int EVP_MD_get_pkey_type(const EVP_MD *md) {
+  if (md) {
+    int sig_nid = 0;
+    if (OBJ_find_sigid_by_algs(&sig_nid, md->type, NID_rsaEncryption)) {
+      return sig_nid;
+    }
+  }
+  return 0;
+}
+
+int EVP_MD_pkey_type(const EVP_MD *md){
+  return EVP_MD_get_pkey_type(md);
+}
+
+const char *EVP_MD_get0_name(const EVP_MD *md) {
+  if (md != NULL) {
+    return OBJ_nid2sn(EVP_MD_nid(md));
+  }
+  return NULL;
+}
+
+const char *EVP_MD_name(const EVP_MD *md) {
+  return EVP_MD_get0_name(md);
+}
+
 extern const EVP_PKEY_ASN1_METHOD *const *AWSLC_non_fips_pkey_evp_asn1_methods(void);
 
 // evp_pkey_asn1_find returns the ASN.1 method table for the given |nid|, which
@@ -242,6 +266,48 @@ int EVP_PKEY_type(int nid) {
   return meth->pkey_id;
 }
 
+EVP_PKEY *EVP_PKEY_new_mac_key(int type, ENGINE *engine, const uint8_t *mac_key,
+                               size_t mac_key_len) {
+  // Only |EVP_PKEY_HMAC| is supported as of now.
+  if (type != EVP_PKEY_HMAC) {
+    OPENSSL_PUT_ERROR(EVP, EVP_R_OPERATION_NOT_SUPPORTED_FOR_THIS_KEYTYPE);
+    return NULL;
+  }
+  // NULL |mac_key| will result in a complete zero-key being used, but in that
+  // case, the length must be zero.
+  if (mac_key == NULL && mac_key_len > 0) {
+    return NULL;
+  }
+
+  EVP_PKEY *ret = EVP_PKEY_new();
+  if (ret == NULL) {
+    OPENSSL_PUT_ERROR(EVP, ERR_LIB_EVP);
+    return NULL;
+  }
+
+  HMAC_KEY *key = HMAC_KEY_new();
+  if(key == NULL) {
+    goto err;
+  }
+  key->key = OPENSSL_memdup(mac_key, mac_key_len);
+  if (key->key == NULL && mac_key_len > 0) {
+    OPENSSL_free(key);
+    goto err;
+  }
+  key->key_len = mac_key_len;
+
+  if(!EVP_PKEY_assign(ret, EVP_PKEY_HMAC, key)) {
+    OPENSSL_free(key);
+    goto err;
+  }
+  return ret;
+
+err:
+  OPENSSL_PUT_ERROR(EVP, ERR_LIB_EVP);
+  EVP_PKEY_free(ret);
+  return NULL;
+}
+
 int EVP_PKEY_set1_RSA(EVP_PKEY *pkey, RSA *key) {
   if (EVP_PKEY_assign_RSA(pkey, key)) {
     RSA_up_ref(key);
