@@ -132,7 +132,9 @@ static int xp_cmp(const X509_PURPOSE *const *a, const X509_PURPOSE *const *b) {
 
 // As much as I'd like to make X509_check_purpose use a "const" X509* I
 // really can't because it does recalculate hashes and do other non-const
-// things.
+// things. If |id| is -1 it just calls |x509v3_cache_extensions| for its
+// side-effect.
+// Returns 1 on success, 0 if x does not allow purpose, -1 on (internal) error.
 int X509_check_purpose(X509 *x, int id, int ca) {
   int idx;
   const X509_PURPOSE *pt;
@@ -200,7 +202,7 @@ int X509_PURPOSE_get_by_id(int purpose) {
     return -1;
   }
 
-  if (!sk_X509_PURPOSE_find(xptable, &idx, &tmp)) {
+  if (!sk_X509_PURPOSE_find_awslc(xptable, &idx, &tmp)) {
     return -1;
   }
   return idx + X509_PURPOSE_COUNT;
@@ -351,23 +353,11 @@ int X509_supported_extension(const X509_EXTENSION *ex) {
 }
 
 static int setup_dp(X509 *x, DIST_POINT *dp) {
-  X509_NAME *iname = NULL;
-  size_t i;
-  if (dp->reasons) {
-    if (dp->reasons->length > 0) {
-      dp->dp_reasons = dp->reasons->data[0];
-    }
-    if (dp->reasons->length > 1) {
-      dp->dp_reasons |= (dp->reasons->data[1] << 8);
-    }
-    dp->dp_reasons &= CRLDP_ALL_REASONS;
-  } else {
-    dp->dp_reasons = CRLDP_ALL_REASONS;
-  }
   if (!dp->distpoint || (dp->distpoint->type != 1)) {
     return 1;
   }
-  for (i = 0; i < sk_GENERAL_NAME_num(dp->CRLissuer); i++) {
+  X509_NAME *iname = NULL;
+  for (size_t i = 0; i < sk_GENERAL_NAME_num(dp->CRLissuer); i++) {
     GENERAL_NAME *gen = sk_GENERAL_NAME_value(dp->CRLissuer, i);
     if (gen->type == GEN_DIRNAME) {
       iname = gen->d.directoryName;
@@ -554,9 +544,6 @@ int x509v3_cache_extensions(X509 *x) {
 
   for (j = 0; j < X509_get_ext_count(x); j++) {
     const X509_EXTENSION *ex = X509_get_ext(x, j);
-    if (OBJ_obj2nid(X509_EXTENSION_get_object(ex)) == NID_freshest_crl) {
-      x->ex_flags |= EXFLAG_FRESHEST;
-    }
     if (!X509_EXTENSION_get_critical(ex)) {
       continue;
     }
@@ -565,6 +552,9 @@ int x509v3_cache_extensions(X509 *x) {
       break;
     }
   }
+  if (!x509_init_signature_info(x)) {
+    x->ex_flags |= EXFLAG_INVALID;
+  }
   x->ex_flags |= EXFLAG_SET;
 
   CRYPTO_MUTEX_unlock_write(&x->lock);
