@@ -85,11 +85,14 @@ OPENSSL_EXPORT const EVP_MD *EVP_sha224(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha256(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha384(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha512(void);
+OPENSSL_EXPORT const EVP_MD *EVP_sha512_224(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha512_256(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha3_224(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha3_256(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha3_384(void);
 OPENSSL_EXPORT const EVP_MD *EVP_sha3_512(void);
+OPENSSL_EXPORT const EVP_MD *EVP_shake128(void);
+OPENSSL_EXPORT const EVP_MD *EVP_shake256(void);
 OPENSSL_EXPORT const EVP_MD *EVP_blake2b256(void);
 
 // EVP_md5_sha1 is a TLS-specific |EVP_MD| which computes the concatenation of
@@ -190,9 +193,10 @@ OPENSSL_EXPORT int EVP_DigestFinal(EVP_MD_CTX *ctx, uint8_t *md_out,
 // bytes from |data| and writes the digest to |md_out|. |EVP_MD_CTX_size| bytes
 // are written, which is at most |EVP_MAX_MD_SIZE|. If |out_size| is not NULL
 // then |*out_size| is set to the number of bytes written. It returns one on
-// success and zero otherwise.
+// success and zero otherwise. If |type| is an XOF, |out_size| must be set to
+// the desired output length.
 OPENSSL_EXPORT int EVP_Digest(const void *data, size_t len, uint8_t *md_out,
-                              unsigned int *md_out_size, const EVP_MD *type,
+                              unsigned int *out_size, const EVP_MD *type,
                               ENGINE *impl);
 
 
@@ -220,8 +224,7 @@ OPENSSL_EXPORT size_t EVP_MD_block_size(const EVP_MD *md);
 #define EVP_MD_FLAG_DIGALGID_ABSENT 2
 
 // EVP_MD_FLAG_XOF indicates that the digest is an extensible-output function
-// (XOF). This flag is defined for compatibility and will never be set in any
-// |EVP_MD| in BoringSSL.
+// (XOF).
 #define EVP_MD_FLAG_XOF 4
 
 
@@ -289,8 +292,8 @@ OPENSSL_EXPORT EVP_MD_CTX *EVP_MD_CTX_create(void);
 // EVP_MD_CTX_destroy calls |EVP_MD_CTX_free|.
 OPENSSL_EXPORT void EVP_MD_CTX_destroy(EVP_MD_CTX *ctx);
 
-// EVP_DigestFinalXOF returns zero and adds an error to the error queue.
-// BoringSSL does not support any XOF digests.
+// EVP_DigestFinalXOF behaves like |EVP_DigestFinal| for XOF digests, writing
+// |len| bytes of extended output to |out|.
 OPENSSL_EXPORT int EVP_DigestFinalXOF(EVP_MD_CTX *ctx, uint8_t *out,
                                       size_t len);
 
@@ -335,6 +338,14 @@ struct env_md_ctx_st {
   // context.
   void *md_data;
 
+  // update is usually copied from |digest->update|. This is only different
+  // when consumed through |EVP_PKEY_HMAC|.
+  // TODO: Look into untangling this, so that |EVP_PKEY_HMAC| can directly call
+  // |digest->update|. |digest->update| operates against |md_data| above, but
+  // |HMAC_CTX| maintains its own data state in |HMAC_CTX->md_ctx|.
+  // |HMAC_Update| also has an additional state transition to handle.
+  void (*update)(EVP_MD_CTX *ctx, const void *data, size_t count);
+
   // pctx is an opaque (at this layer) pointer to additional context that
   // EVP_PKEY functions may store in this object.
   EVP_PKEY_CTX *pctx;
@@ -343,9 +354,11 @@ struct env_md_ctx_st {
   // manipulate |pctx|.
   const struct evp_md_pctx_ops *pctx_ops;
 
-  // flags, only used to set flag |EVP_MD_CTX_FLAG_KEEP_PKEY_CTX|, so as to let
-  // |*pctx| refrain from being freed when |*pctx| was set externally with
-  // |EVP_MD_CTX_set_pkey_ctx|.
+  // flags is only used for two cases.
+  // 1. Set flag |EVP_MD_CTX_FLAG_KEEP_PKEY_CTX|, so as to let |*pctx| refrain
+  //    from being freed when |*pctx| was set externally with
+  //    |EVP_MD_CTX_set_pkey_ctx|.
+  // 2. Set flag |EVP_MD_CTX_HMAC| for |EVP_PKEY_HMAC|.
   unsigned long flags;
 } /* EVP_MD_CTX */;
 
