@@ -71,6 +71,9 @@ typedef enum {
     S2N_ERR_ENCRYPT = S2N_ERR_T_PROTO_START,
     S2N_ERR_DECRYPT,
     S2N_ERR_BAD_MESSAGE,
+    S2N_ERR_UNEXPECTED_CERT_REQUEST,
+    S2N_ERR_MISSING_CERT_REQUEST,
+    S2N_ERR_MISSING_CLIENT_CERT,
     S2N_ERR_KEY_INIT,
     S2N_ERR_KEY_DESTROY,
     S2N_ERR_DH_SERIALIZING,
@@ -94,6 +97,7 @@ typedef enum {
     S2N_ERR_INVALID_HELLO_RETRY,
     S2N_ERR_INVALID_SIGNATURE_ALGORITHM,
     S2N_ERR_INVALID_SIGNATURE_SCHEME,
+    S2N_ERR_NO_VALID_SIGNATURE_SCHEME,
     S2N_ERR_CBC_VERIFY,
     S2N_ERR_DH_COPYING_PUBLIC_KEY,
     S2N_ERR_SIGN,
@@ -101,6 +105,8 @@ typedef enum {
     S2N_ERR_ECDHE_GEN_KEY,
     S2N_ERR_ECDHE_SHARED_SECRET,
     S2N_ERR_ECDHE_UNSUPPORTED_CURVE,
+    S2N_ERR_ECDHE_INVALID_PUBLIC_KEY,
+    S2N_ERR_ECDHE_INVALID_PUBLIC_KEY_FIPS,
     S2N_ERR_ECDSA_UNSUPPORTED_CURVE,
     S2N_ERR_ECDHE_SERIALIZING,
     S2N_ERR_KEM_UNSUPPORTED_PARAMS,
@@ -141,6 +147,7 @@ typedef enum {
     S2N_ERR_MAX_EARLY_DATA_SIZE,
     S2N_ERR_EARLY_DATA_TRIAL_DECRYPT,
     S2N_ERR_NO_RENEGOTIATION,
+    S2N_ERR_KTLS_KEYUPDATE,
     S2N_ERR_T_PROTO_END,
 
     /* S2N_ERR_T_INTERNAL */
@@ -222,7 +229,6 @@ typedef enum {
     S2N_ERR_ASYNC_CALLBACK_FAILED,
     S2N_ERR_ASYNC_MORE_THAN_ONE,
     S2N_ERR_PQ_CRYPTO,
-    S2N_ERR_PQ_DISABLED,
     S2N_ERR_INVALID_CERT_STATE,
     S2N_ERR_INVALID_EARLY_DATA_STATE,
     S2N_ERR_PKEY_CTX_INIT,
@@ -319,6 +325,10 @@ typedef enum {
     S2N_ERR_KTLS_BAD_CMSG,
     S2N_ERR_KTLS_RENEG,
     S2N_ERR_ATOMIC,
+    S2N_ERR_KTLS_KEY_LIMIT,
+    S2N_ERR_SECURITY_POLICY_INCOMPATIBLE_CERT,
+    S2N_ERR_INVALID_SERIALIZED_CONNECTION,
+    S2N_ERR_TOO_MANY_CAS,
     S2N_ERR_T_USAGE_END,
 } s2n_error;
 
