@@ -14,19 +14,21 @@
  */
 
 #include "error/s2n_errno.h"
-
 #include "stuffer/s2n_stuffer.h"
-
 #include "utils/s2n_annotations.h"
 #include "utils/s2n_safety.h"
 
 /* Writes length bytes of input to stuffer, in network order, starting from the smallest byte of input. */
 int s2n_stuffer_write_network_order(struct s2n_stuffer *stuffer, const uint64_t input, const uint8_t length)
 {
+    if (length == 0) {
+        return S2N_SUCCESS;
+    }
+    POSIX_ENSURE_REF(stuffer);
     POSIX_ENSURE(length <= sizeof(input), S2N_ERR_SAFETY);
     POSIX_GUARD(s2n_stuffer_skip_write(stuffer, length));
-    uint8_t *data = (stuffer->blob.data) ? (stuffer->blob.data + stuffer->write_cursor - length) : NULL;
-
+    POSIX_ENSURE_REF(stuffer->blob.data);
+    uint8_t *data = stuffer->blob.data + stuffer->write_cursor - length;
     for (int i = 0; i < length; i++) {
         S2N_INVARIANT(i <= length);
         uint8_t shift = (length - i - 1) * CHAR_BIT;
@@ -41,7 +43,9 @@ int s2n_stuffer_reserve(struct s2n_stuffer *stuffer, struct s2n_stuffer_reservat
     POSIX_PRECONDITION(s2n_stuffer_validate(stuffer));
     POSIX_ENSURE_REF(reservation);
 
-    *reservation = (struct s2n_stuffer_reservation) {.stuffer = stuffer, .write_cursor = stuffer->write_cursor, .length = length};
+    *reservation = (struct s2n_stuffer_reservation){ .stuffer = stuffer,
+        .write_cursor = stuffer->write_cursor,
+        .length = length };
 
     POSIX_GUARD(s2n_stuffer_skip_write(stuffer, reservation->length));
     POSIX_CHECKED_MEMSET(stuffer->blob.data + reservation->write_cursor, S2N_WIPE_PATTERN, reservation->length);
@@ -49,7 +53,7 @@ int s2n_stuffer_reserve(struct s2n_stuffer *stuffer, struct s2n_stuffer_reservat
     return S2N_SUCCESS;
 }
 
-int s2n_stuffer_read_uint8(struct s2n_stuffer *stuffer, uint8_t * u)
+int s2n_stuffer_read_uint8(struct s2n_stuffer *stuffer, uint8_t *u)
 {
     POSIX_GUARD(s2n_stuffer_read_bytes(stuffer, u, sizeof(uint8_t)));
 
@@ -68,7 +72,7 @@ int s2n_stuffer_reserve_uint8(struct s2n_stuffer *stuffer, struct s2n_stuffer_re
     return s2n_stuffer_reserve(stuffer, reservation, sizeof(uint8_t));
 }
 
-int s2n_stuffer_read_uint16(struct s2n_stuffer *stuffer, uint16_t * u)
+int s2n_stuffer_read_uint16(struct s2n_stuffer *stuffer, uint16_t *u)
 {
     POSIX_ENSURE_REF(u);
     uint8_t data[sizeof(uint16_t)];
@@ -91,7 +95,7 @@ int s2n_stuffer_reserve_uint16(struct s2n_stuffer *stuffer, struct s2n_stuffer_r
     return s2n_stuffer_reserve(stuffer, reservation, sizeof(uint16_t));
 }
 
-int s2n_stuffer_read_uint24(struct s2n_stuffer *stuffer, uint32_t * u)
+int s2n_stuffer_read_uint24(struct s2n_stuffer *stuffer, uint32_t *u)
 {
     POSIX_ENSURE_REF(u);
     uint8_t data[SIZEOF_UINT24];
@@ -115,7 +119,7 @@ int s2n_stuffer_reserve_uint24(struct s2n_stuffer *stuffer, struct s2n_stuffer_r
     return s2n_stuffer_reserve(stuffer, reservation, SIZEOF_UINT24);
 }
 
-int s2n_stuffer_read_uint32(struct s2n_stuffer *stuffer, uint32_t * u)
+int s2n_stuffer_read_uint32(struct s2n_stuffer *stuffer, uint32_t *u)
 {
     POSIX_ENSURE_REF(u);
     uint8_t data[sizeof(uint32_t)];
@@ -135,7 +139,7 @@ int s2n_stuffer_write_uint32(struct s2n_stuffer *stuffer, const uint32_t u)
     return s2n_stuffer_write_network_order(stuffer, u, sizeof(u));
 }
 
-int s2n_stuffer_read_uint64(struct s2n_stuffer *stuffer, uint64_t * u)
+int s2n_stuffer_read_uint64(struct s2n_stuffer *stuffer, uint64_t *u)
 {
     POSIX_ENSURE_REF(u);
     uint8_t data[sizeof(uint64_t)];
@@ -166,13 +170,14 @@ static int length_matches_value_check(uint32_t value, uint8_t length)
 
     if (length < sizeof(uint32_t)) {
         /* Value should be less than the maximum for its length */
-        POSIX_ENSURE(value < (0x01 << (length * 8)), S2N_ERR_SIZE_MISMATCH);
+        const uint32_t size_max = 1 << (length * 8);
+        POSIX_ENSURE(value < size_max, S2N_ERR_SIZE_MISMATCH);
     }
 
     return S2N_SUCCESS;
 }
 
-static int s2n_stuffer_write_reservation_impl(struct s2n_stuffer_reservation* reservation, const uint32_t u)
+static int s2n_stuffer_write_reservation_impl(struct s2n_stuffer_reservation *reservation, const uint32_t u)
 {
     reservation->stuffer->write_cursor = reservation->write_cursor;
     POSIX_PRECONDITION(s2n_stuffer_validate(reservation->stuffer));
@@ -183,7 +188,7 @@ static int s2n_stuffer_write_reservation_impl(struct s2n_stuffer_reservation* re
     return S2N_SUCCESS;
 }
 
-int s2n_stuffer_write_reservation(struct s2n_stuffer_reservation* reservation, const uint32_t u)
+int s2n_stuffer_write_reservation(struct s2n_stuffer_reservation *reservation, const uint32_t u)
 {
     POSIX_PRECONDITION(s2n_stuffer_reservation_validate(reservation));
     uint32_t old_write_cursor = reservation->stuffer->write_cursor;
@@ -192,7 +197,7 @@ int s2n_stuffer_write_reservation(struct s2n_stuffer_reservation* reservation, c
     return result;
 }
 
-int s2n_stuffer_write_vector_size(struct s2n_stuffer_reservation* reservation)
+int s2n_stuffer_write_vector_size(struct s2n_stuffer_reservation *reservation)
 {
     POSIX_PRECONDITION(s2n_stuffer_reservation_validate(reservation));
     uint32_t size = 0;
