@@ -13,19 +13,16 @@
  * permissions and limitations under the License.
  */
 
+#include "tls/extensions/s2n_server_max_fragment_length.h"
+
 #include <sys/param.h>
 
 #include "error/s2n_errno.h"
-
 #include "stuffer/s2n_stuffer.h"
-
-#include "utils/s2n_safety.h"
-
+#include "tls/s2n_connection.h"
 #include "tls/s2n_tls.h"
 #include "tls/s2n_tls_parameters.h"
-#include "tls/s2n_connection.h"
-
-#include "tls/extensions/s2n_server_max_fragment_length.h"
+#include "utils/s2n_safety.h"
 
 static bool s2n_max_fragment_length_should_send(struct s2n_connection *conn);
 static int s2n_max_fragment_length_send(struct s2n_connection *conn, struct s2n_stuffer *out);
@@ -57,7 +54,7 @@ static int s2n_max_fragment_length_recv(struct s2n_connection *conn, struct s2n_
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->config);
 
-    uint8_t mfl_code;
+    uint8_t mfl_code = 0;
     POSIX_GUARD(s2n_stuffer_read_uint8(extension, &mfl_code));
 
     /*
