@@ -23,12 +23,10 @@
 #include "tls/s2n_record.h"
 #include "tls/s2n_resume.h"
 #include "tls/s2n_tls_parameters.h"
+#include "utils/s2n_atomic.h"
 #include "utils/s2n_blob.h"
 #include "utils/s2n_safety.h"
 
-#define S2N_TLS_ALERT_LEVEL_WARNING 1
-#define S2N_TLS_ALERT_LEVEL_FATAL   2
-
 #define S2N_ALERT_CASE(error, alert_code) \
     case (error):                         \
         *alert = (alert_code);            \
@@ -44,6 +42,8 @@ static S2N_RESULT s2n_translate_protocol_error_to_alert(int error_code, uint8_t
 
     switch (error_code) {
         S2N_ALERT_CASE(S2N_ERR_MISSING_EXTENSION, S2N_TLS_ALERT_MISSING_EXTENSION);
+        S2N_ALERT_CASE(S2N_ERR_NO_VALID_SIGNATURE_SCHEME, S2N_TLS_ALERT_HANDSHAKE_FAILURE);
+        S2N_ALERT_CASE(S2N_ERR_MISSING_CLIENT_CERT, S2N_TLS_ALERT_CERTIFICATE_REQUIRED);
 
         /* TODO: The ERR_BAD_MESSAGE -> ALERT_UNEXPECTED_MESSAGE mapping
          * isn't always correct. Sometimes s2n-tls uses ERR_BAD_MESSAGE
@@ -52,6 +52,8 @@ static S2N_RESULT s2n_translate_protocol_error_to_alert(int error_code, uint8_t
          * our errors should be equally or more specific than alerts, not less.
          */
         S2N_ALERT_CASE(S2N_ERR_BAD_MESSAGE, S2N_TLS_ALERT_UNEXPECTED_MESSAGE);
+        S2N_ALERT_CASE(S2N_ERR_UNEXPECTED_CERT_REQUEST, S2N_TLS_ALERT_UNEXPECTED_MESSAGE);
+        S2N_ALERT_CASE(S2N_ERR_MISSING_CERT_REQUEST, S2N_TLS_ALERT_UNEXPECTED_MESSAGE);
 
         /* For errors involving secure renegotiation:
          *= https://tools.ietf.org/rfc/rfc5746#3.4
@@ -61,6 +63,8 @@ static S2N_RESULT s2n_translate_protocol_error_to_alert(int error_code, uint8_t
          */
         S2N_ALERT_CASE(S2N_ERR_NO_RENEGOTIATION, S2N_TLS_ALERT_HANDSHAKE_FAILURE);
 
+        S2N_ALERT_CASE(S2N_ERR_KTLS_KEYUPDATE, S2N_TLS_ALERT_UNEXPECTED_MESSAGE);
+
         /* TODO: Add mappings for other protocol errors.
          */
         S2N_NO_ALERT(S2N_ERR_ENCRYPT);
@@ -95,6 +99,8 @@ static S2N_RESULT s2n_translate_protocol_error_to_alert(int error_code, uint8_t
         S2N_NO_ALERT(S2N_ERR_ECDHE_GEN_KEY);
         S2N_NO_ALERT(S2N_ERR_ECDHE_SHARED_SECRET);
         S2N_NO_ALERT(S2N_ERR_ECDHE_UNSUPPORTED_CURVE);
+        S2N_NO_ALERT(S2N_ERR_ECDHE_INVALID_PUBLIC_KEY);
+        S2N_NO_ALERT(S2N_ERR_ECDHE_INVALID_PUBLIC_KEY_FIPS);
         S2N_NO_ALERT(S2N_ERR_ECDSA_UNSUPPORTED_CURVE);
         S2N_NO_ALERT(S2N_ERR_ECDHE_SERIALIZING);
         S2N_NO_ALERT(S2N_ERR_KEM_UNSUPPORTED_PARAMS);
@@ -104,10 +110,12 @@ static S2N_RESULT s2n_translate_protocol_error_to_alert(int error_code, uint8_t
         S2N_NO_ALERT(S2N_ERR_RECORD_LIMIT);
         S2N_NO_ALERT(S2N_ERR_CERT_UNTRUSTED);
         S2N_NO_ALERT(S2N_ERR_CERT_REVOKED);
+        S2N_NO_ALERT(S2N_ERR_CERT_NOT_YET_VALID);
         S2N_NO_ALERT(S2N_ERR_CERT_EXPIRED);
         S2N_NO_ALERT(S2N_ERR_CERT_TYPE_UNSUPPORTED);
         S2N_NO_ALERT(S2N_ERR_CERT_INVALID);
         S2N_NO_ALERT(S2N_ERR_CERT_MAX_CHAIN_DEPTH_EXCEEDED);
+        S2N_NO_ALERT(S2N_ERR_CERT_REJECTED);
         S2N_NO_ALERT(S2N_ERR_CRL_LOOKUP_FAILED);
         S2N_NO_ALERT(S2N_ERR_CRL_SIGNATURE);
         S2N_NO_ALERT(S2N_ERR_CRL_ISSUER);
@@ -143,6 +151,11 @@ static bool s2n_alerts_supported(struct s2n_connection *conn)
     return !s2n_connection_is_quic_enabled(conn);
 }
 
+/* In TLS1.3 all Alerts
+ *= https://tools.ietf.org/rfc/rfc8446#section-6
+ *# MUST be treated as error alerts when received
+ *# regardless of the AlertLevel in the message.
+ */
 static bool s2n_process_as_warning(struct s2n_connection *conn, uint8_t level, uint8_t type)
 {
     /* Only TLS1.2 considers the alert level. The alert level field is
@@ -158,20 +171,6 @@ static bool s2n_process_as_warning(struct s2n_connection *conn, uint8_t level, u
     return type == S2N_TLS_ALERT_USER_CANCELED;
 }
 
-S2N_RESULT s2n_alerts_close_if_fatal(struct s2n_connection *conn, struct s2n_blob *alert)
-{
-    RESULT_ENSURE_REF(conn);
-    RESULT_ENSURE_REF(alert);
-    RESULT_ENSURE_EQ(alert->size, S2N_ALERT_LENGTH);
-    /* Only one alert should currently be treated as a warning */
-    if (alert->data[1] == S2N_TLS_ALERT_NO_RENEGOTIATION) {
-        RESULT_ENSURE_EQ(alert->data[0], S2N_TLS_ALERT_LEVEL_WARNING);
-        return S2N_RESULT_OK;
-    }
-    conn->closing = true;
-    return S2N_RESULT_OK;
-}
-
 int s2n_error_get_alert(int error, uint8_t *alert)
 {
     int error_type = s2n_error_get_type(error);
@@ -220,8 +219,8 @@ int s2n_process_alert_fragment(struct s2n_connection *conn)
         if (s2n_stuffer_data_available(&conn->alert_in) == 2) {
             /* Close notifications are handled as shutdowns */
             if (conn->alert_in_data[1] == S2N_TLS_ALERT_CLOSE_NOTIFY) {
-                conn->closed = 1;
-                conn->close_notify_received = true;
+                s2n_atomic_flag_set(&conn->read_closed);
+                s2n_atomic_flag_set(&conn->close_notify_received);
                 return 0;
             }
 
@@ -236,8 +235,13 @@ int s2n_process_alert_fragment(struct s2n_connection *conn)
                 conn->config->cache_delete(conn, conn->config->cache_delete_data, conn->session_id, conn->session_id_len);
             }
 
-            /* All other alerts are treated as fatal errors */
-            conn->closed = 1;
+            /* All other alerts are treated as fatal errors.
+             *
+             *= https://tools.ietf.org/rfc/rfc8446#section-6
+             *# Unknown Alert types MUST be treated as error alerts.
+             */
+            POSIX_GUARD_RESULT(s2n_connection_set_closed(conn));
+            s2n_atomic_flag_set(&conn->error_alert_received);
             POSIX_BAIL(S2N_ERR_ALERT);
         }
     }
@@ -245,63 +249,25 @@ int s2n_process_alert_fragment(struct s2n_connection *conn)
     return 0;
 }
 
-int s2n_queue_writer_close_alert_warning(struct s2n_connection *conn)
+static S2N_RESULT s2n_queue_reader_alert(struct s2n_connection *conn, s2n_tls_alert_code code)
 {
-    POSIX_ENSURE_REF(conn);
-
-    uint8_t alert[2];
-    alert[0] = S2N_TLS_ALERT_LEVEL_WARNING;
-    alert[1] = S2N_TLS_ALERT_CLOSE_NOTIFY;
-
-    struct s2n_blob out = { .data = alert, .size = sizeof(alert) };
-
-    /* If there is an alert pending or we've already sent a close_notify, do nothing */
-    if (s2n_stuffer_data_available(&conn->writer_alert_out) || conn->close_notify_queued) {
-        return S2N_SUCCESS;
+    RESULT_ENSURE_REF(conn);
+    if (!conn->reader_alert_out) {
+        conn->reader_alert_out = code;
     }
-
-    if (!s2n_alerts_supported(conn)) {
-        return S2N_SUCCESS;
-    }
-
-    POSIX_GUARD(s2n_stuffer_write(&conn->writer_alert_out, &out));
-    conn->close_notify_queued = 1;
-
-    return S2N_SUCCESS;
-}
-
-static int s2n_queue_reader_alert(struct s2n_connection *conn, uint8_t level, uint8_t error_code)
-{
-    POSIX_ENSURE_REF(conn);
-
-    uint8_t alert[2];
-    alert[0] = level;
-    alert[1] = error_code;
-
-    struct s2n_blob out = { .data = alert, .size = sizeof(alert) };
-
-    /* If there is an alert pending, do nothing */
-    if (s2n_stuffer_data_available(&conn->reader_alert_out)) {
-        return S2N_SUCCESS;
-    }
-
-    if (!s2n_alerts_supported(conn)) {
-        return S2N_SUCCESS;
-    }
-
-    POSIX_GUARD(s2n_stuffer_write(&conn->reader_alert_out, &out));
-
-    return S2N_SUCCESS;
+    return S2N_RESULT_OK;
 }
 
 int s2n_queue_reader_unsupported_protocol_version_alert(struct s2n_connection *conn)
 {
-    return s2n_queue_reader_alert(conn, S2N_TLS_ALERT_LEVEL_FATAL, S2N_TLS_ALERT_PROTOCOL_VERSION);
+    POSIX_GUARD_RESULT(s2n_queue_reader_alert(conn, S2N_TLS_ALERT_PROTOCOL_VERSION));
+    return S2N_SUCCESS;
 }
 
 int s2n_queue_reader_handshake_failure_alert(struct s2n_connection *conn)
 {
-    return s2n_queue_reader_alert(conn, S2N_TLS_ALERT_LEVEL_FATAL, S2N_TLS_ALERT_HANDSHAKE_FAILURE);
+    POSIX_GUARD_RESULT(s2n_queue_reader_alert(conn, S2N_TLS_ALERT_HANDSHAKE_FAILURE));
+    return S2N_SUCCESS;
 }
 
 S2N_RESULT s2n_queue_reader_no_renegotiation_alert(struct s2n_connection *conn)
@@ -314,10 +280,72 @@ S2N_RESULT s2n_queue_reader_no_renegotiation_alert(struct s2n_connection *conn)
      *# handshake_failure alert.
      **/
     if (s2n_connection_get_protocol_version(conn) == S2N_SSLv3) {
-        RESULT_GUARD_POSIX(s2n_queue_reader_alert(conn, S2N_TLS_ALERT_LEVEL_FATAL, S2N_TLS_ALERT_HANDSHAKE_FAILURE));
+        RESULT_GUARD_POSIX(s2n_queue_reader_handshake_failure_alert(conn));
+        RESULT_BAIL(S2N_ERR_BAD_MESSAGE);
+    }
+
+    if (!conn->reader_warning_out) {
+        conn->reader_warning_out = S2N_TLS_ALERT_NO_RENEGOTIATION;
+    }
+    return S2N_RESULT_OK;
+}
+
+S2N_RESULT s2n_alerts_write_error_or_close_notify(struct s2n_connection *conn)
+{
+    if (!s2n_alerts_supported(conn)) {
         return S2N_RESULT_OK;
     }
 
-    RESULT_GUARD_POSIX(s2n_queue_reader_alert(conn, S2N_TLS_ALERT_LEVEL_WARNING, S2N_TLS_ALERT_NO_RENEGOTIATION));
+    /*
+     *= https://tools.ietf.org/rfc/rfc8446#section-6.2
+     *= type=exception
+     *= reason=Specific alerts could expose a side-channel attack vector.
+     *# The phrases "terminate the connection with an X
+     *# alert" and "abort the handshake with an X alert" mean that the
+     *# implementation MUST send alert X if it sends any alert.
+     *
+     * By default, s2n-tls sends a generic close_notify alert, even in
+     * response to fatal errors. This is done to avoid potential
+     * side-channel attacks since specific alerts could reveal information
+     * about why the error occured.
+     */
+    uint8_t code = S2N_TLS_ALERT_CLOSE_NOTIFY;
+    uint8_t level = S2N_TLS_ALERT_LEVEL_WARNING;
+
+    /* s2n-tls sends a very small subset of more specific error alerts.
+     * Since either the reader or the writer can produce one of these alerts,
+     * but only a single alert can be reported, we prioritize writer alerts.
+     */
+    if (conn->writer_alert_out) {
+        code = conn->writer_alert_out;
+        level = S2N_TLS_ALERT_LEVEL_FATAL;
+    } else if (conn->reader_alert_out) {
+        code = conn->reader_alert_out;
+        level = S2N_TLS_ALERT_LEVEL_FATAL;
+    }
+
+    struct s2n_blob alert = { 0 };
+    uint8_t alert_bytes[] = { level, code };
+    RESULT_GUARD_POSIX(s2n_blob_init(&alert, alert_bytes, sizeof(alert_bytes)));
+
+    RESULT_GUARD(s2n_record_write(conn, TLS_ALERT, &alert));
+    conn->alert_sent = true;
+    return S2N_RESULT_OK;
+}
+
+S2N_RESULT s2n_alerts_write_warning(struct s2n_connection *conn)
+{
+    if (!s2n_alerts_supported(conn)) {
+        return S2N_RESULT_OK;
+    }
+
+    uint8_t code = conn->reader_warning_out;
+    uint8_t level = S2N_TLS_ALERT_LEVEL_WARNING;
+
+    struct s2n_blob alert = { 0 };
+    uint8_t alert_bytes[] = { level, code };
+    RESULT_GUARD_POSIX(s2n_blob_init(&alert, alert_bytes, sizeof(alert_bytes)));
+
+    RESULT_GUARD(s2n_record_write(conn, TLS_ALERT, &alert));
     return S2N_RESULT_OK;
 }
