@@ -18,8 +18,8 @@
 
 #include "crypto/s2n_cipher.h"
 #include "crypto/s2n_openssl.h"
+#include "crypto/s2n_pq.h"
 #include "error/s2n_errno.h"
-#include "pq-crypto/s2n_pq.h"
 #include "tls/s2n_auth_selection.h"
 #include "tls/s2n_kex.h"
 #include "tls/s2n_psk.h"
@@ -210,6 +210,7 @@ const struct s2n_record_algorithm s2n_tls13_record_alg_chacha20_poly1305 = {
 struct s2n_cipher_suite s2n_null_cipher_suite = {
     .available = 1,
     .name = "TLS_NULL_WITH_NULL_NULL",
+    .iana_name = "TLS_NULL_WITH_NULL_NULL",
     .iana_value = { TLS_NULL_WITH_NULL_NULL },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -219,6 +220,7 @@ struct s2n_cipher_suite s2n_null_cipher_suite = {
 struct s2n_cipher_suite s2n_rsa_with_rc4_128_md5 = /* 0x00,0x04 */ {
     .available = 0,
     .name = "RC4-MD5",
+    .iana_name = "TLS_RSA_WITH_RC4_128_MD5",
     .iana_value = { TLS_RSA_WITH_RC4_128_MD5 },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -233,6 +235,7 @@ struct s2n_cipher_suite s2n_rsa_with_rc4_128_md5 = /* 0x00,0x04 */ {
 struct s2n_cipher_suite s2n_rsa_with_rc4_128_sha = /* 0x00,0x05 */ {
     .available = 0,
     .name = "RC4-SHA",
+    .iana_name = "TLS_RSA_WITH_RC4_128_SHA",
     .iana_value = { TLS_RSA_WITH_RC4_128_SHA },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -247,6 +250,7 @@ struct s2n_cipher_suite s2n_rsa_with_rc4_128_sha = /* 0x00,0x05 */ {
 struct s2n_cipher_suite s2n_rsa_with_3des_ede_cbc_sha = /* 0x00,0x0A */ {
     .available = 0,
     .name = "DES-CBC3-SHA",
+    .iana_name = "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
     .iana_value = { TLS_RSA_WITH_3DES_EDE_CBC_SHA },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -261,6 +265,7 @@ struct s2n_cipher_suite s2n_rsa_with_3des_ede_cbc_sha = /* 0x00,0x0A */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_3des_ede_cbc_sha = /* 0x00,0x16 */ {
     .available = 0,
     .name = "DHE-RSA-DES-CBC3-SHA",
+    .iana_name = "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA",
     .iana_value = { TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -275,6 +280,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_3des_ede_cbc_sha = /* 0x00,0x16 */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_128_cbc_sha = /* 0x00,0x2F */ {
     .available = 0,
     .name = "AES128-SHA",
+    .iana_name = "TLS_RSA_WITH_AES_128_CBC_SHA",
     .iana_value = { TLS_RSA_WITH_AES_128_CBC_SHA },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -289,6 +295,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_128_cbc_sha = /* 0x00,0x2F */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_cbc_sha = /* 0x00,0x33 */ {
     .available = 0,
     .name = "DHE-RSA-AES128-SHA",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_128_CBC_SHA",
     .iana_value = { TLS_DHE_RSA_WITH_AES_128_CBC_SHA },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -303,6 +310,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_cbc_sha = /* 0x00,0x33 */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_256_cbc_sha = /* 0x00,0x35 */ {
     .available = 0,
     .name = "AES256-SHA",
+    .iana_name = "TLS_RSA_WITH_AES_256_CBC_SHA",
     .iana_value = { TLS_RSA_WITH_AES_256_CBC_SHA },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -317,6 +325,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_256_cbc_sha = /* 0x00,0x35 */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_cbc_sha = /* 0x00,0x39 */ {
     .available = 0,
     .name = "DHE-RSA-AES256-SHA",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_256_CBC_SHA",
     .iana_value = { TLS_DHE_RSA_WITH_AES_256_CBC_SHA },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -331,6 +340,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_cbc_sha = /* 0x00,0x39 */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_128_cbc_sha256 = /* 0x00,0x3C */ {
     .available = 0,
     .name = "AES128-SHA256",
+    .iana_name = "TLS_RSA_WITH_AES_128_CBC_SHA256",
     .iana_value = { TLS_RSA_WITH_AES_128_CBC_SHA256 },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -345,6 +355,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_128_cbc_sha256 = /* 0x00,0x3C */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_256_cbc_sha256 = /* 0x00,0x3D */ {
     .available = 0,
     .name = "AES256-SHA256",
+    .iana_name = "TLS_RSA_WITH_AES_256_CBC_SHA256",
     .iana_value = { TLS_RSA_WITH_AES_256_CBC_SHA256 },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -359,6 +370,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_256_cbc_sha256 = /* 0x00,0x3D */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_cbc_sha256 = /* 0x00,0x67 */ {
     .available = 0,
     .name = "DHE-RSA-AES128-SHA256",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_128_CBC_SHA256",
     .iana_value = { TLS_DHE_RSA_WITH_AES_128_CBC_SHA256 },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -373,6 +385,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_cbc_sha256 = /* 0x00,0x67 */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_cbc_sha256 = /* 0x00,0x6B */ {
     .available = 0,
     .name = "DHE-RSA-AES256-SHA256",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_256_CBC_SHA256",
     .iana_value = { TLS_DHE_RSA_WITH_AES_256_CBC_SHA256 },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -387,6 +400,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_cbc_sha256 = /* 0x00,0x6B */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_128_gcm_sha256 = /* 0x00,0x9C */ {
     .available = 0,
     .name = "AES128-GCM-SHA256",
+    .iana_name = "TLS_RSA_WITH_AES_128_GCM_SHA256",
     .iana_value = { TLS_RSA_WITH_AES_128_GCM_SHA256 },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -401,6 +415,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_128_gcm_sha256 = /* 0x00,0x9C */ {
 struct s2n_cipher_suite s2n_rsa_with_aes_256_gcm_sha384 = /* 0x00,0x9D */ {
     .available = 0,
     .name = "AES256-GCM-SHA384",
+    .iana_name = "TLS_RSA_WITH_AES_256_GCM_SHA384",
     .iana_value = { TLS_RSA_WITH_AES_256_GCM_SHA384 },
     .key_exchange_alg = &s2n_rsa,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -415,6 +430,7 @@ struct s2n_cipher_suite s2n_rsa_with_aes_256_gcm_sha384 = /* 0x00,0x9D */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_gcm_sha256 = /* 0x00,0x9E */ {
     .available = 0,
     .name = "DHE-RSA-AES128-GCM-SHA256",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_128_GCM_SHA256",
     .iana_value = { TLS_DHE_RSA_WITH_AES_128_GCM_SHA256 },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -429,6 +445,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_128_gcm_sha256 = /* 0x00,0x9E */ {
 struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_gcm_sha384 = /* 0x00,0x9F */ {
     .available = 0,
     .name = "DHE-RSA-AES256-GCM-SHA384",
+    .iana_name = "TLS_DHE_RSA_WITH_AES_256_GCM_SHA384",
     .iana_value = { TLS_DHE_RSA_WITH_AES_256_GCM_SHA384 },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -443,6 +460,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_aes_256_gcm_sha384 = /* 0x00,0x9F */ {
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_cbc_sha = /* 0xC0,0x09 */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES128-SHA",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -457,6 +475,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_cbc_sha = /* 0xC0,0x09 */ {
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_cbc_sha = /* 0xC0,0x0A */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES256-SHA",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -471,6 +490,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_cbc_sha = /* 0xC0,0x0A */ {
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_rc4_128_sha = /* 0xC0,0x11 */ {
     .available = 0,
     .name = "ECDHE-RSA-RC4-SHA",
+    .iana_name = "TLS_ECDHE_RSA_WITH_RC4_128_SHA",
     .iana_value = { TLS_ECDHE_RSA_WITH_RC4_128_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -485,6 +505,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_rc4_128_sha = /* 0xC0,0x11 */ {
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_3des_ede_cbc_sha = /* 0xC0,0x12 */ {
     .available = 0,
     .name = "ECDHE-RSA-DES-CBC3-SHA",
+    .iana_name = "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA",
     .iana_value = { TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -499,6 +520,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_3des_ede_cbc_sha = /* 0xC0,0x12 */ {
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_cbc_sha = /* 0xC0,0x13 */ {
     .available = 0,
     .name = "ECDHE-RSA-AES128-SHA",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -513,6 +535,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_cbc_sha = /* 0xC0,0x13 */ {
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_cbc_sha = /* 0xC0,0x14 */ {
     .available = 0,
     .name = "ECDHE-RSA-AES256-SHA",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -527,6 +550,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_cbc_sha = /* 0xC0,0x14 */ {
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_cbc_sha256 = /* 0xC0,0x23 */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES128-SHA256",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -541,6 +565,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_cbc_sha256 = /* 0xC0,0x23 *
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_cbc_sha384 = /* 0xC0,0x24 */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES256-SHA384",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -555,6 +580,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_cbc_sha384 = /* 0xC0,0x24 *
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_cbc_sha256 = /* 0xC0,0x27 */ {
     .available = 0,
     .name = "ECDHE-RSA-AES128-SHA256",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -569,6 +595,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_cbc_sha256 = /* 0xC0,0x27 */
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_cbc_sha384 = /* 0xC0,0x28 */ {
     .available = 0,
     .name = "ECDHE-RSA-AES256-SHA384",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -583,6 +610,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_cbc_sha384 = /* 0xC0,0x28 */
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_gcm_sha256 = /* 0xC0,0x2B */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES128-GCM-SHA256",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -597,6 +625,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_128_gcm_sha256 = /* 0xC0,0x2B *
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_gcm_sha384 = /* 0xC0,0x2C */ {
     .available = 0,
     .name = "ECDHE-ECDSA-AES256-GCM-SHA384",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -611,6 +640,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_aes_256_gcm_sha384 = /* 0xC0,0x2C *
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_gcm_sha256 = /* 0xC0,0x2F */ {
     .available = 0,
     .name = "ECDHE-RSA-AES128-GCM-SHA256",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -625,6 +655,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_128_gcm_sha256 = /* 0xC0,0x2F */
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_gcm_sha384 = /* 0xC0,0x30 */ {
     .available = 0,
     .name = "ECDHE-RSA-AES256-GCM-SHA384",
+    .iana_name = "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",
     .iana_value = { TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -639,6 +670,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_aes_256_gcm_sha384 = /* 0xC0,0x30 */
 struct s2n_cipher_suite s2n_ecdhe_rsa_with_chacha20_poly1305_sha256 = /* 0xCC,0xA8 */ {
     .available = 0,
     .name = "ECDHE-RSA-CHACHA20-POLY1305",
+    .iana_name = "TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256",
     .iana_value = { TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -653,6 +685,7 @@ struct s2n_cipher_suite s2n_ecdhe_rsa_with_chacha20_poly1305_sha256 = /* 0xCC,0x
 struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_chacha20_poly1305_sha256 = /* 0xCC,0xA9 */ {
     .available = 0,
     .name = "ECDHE-ECDSA-CHACHA20-POLY1305",
+    .iana_name = "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256",
     .iana_value = { TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256 },
     .key_exchange_alg = &s2n_ecdhe,
     .auth_method = S2N_AUTHENTICATION_ECDSA,
@@ -667,6 +700,7 @@ struct s2n_cipher_suite s2n_ecdhe_ecdsa_with_chacha20_poly1305_sha256 = /* 0xCC,
 struct s2n_cipher_suite s2n_dhe_rsa_with_chacha20_poly1305_sha256 = /* 0xCC,0xAA */ {
     .available = 0,
     .name = "DHE-RSA-CHACHA20-POLY1305",
+    .iana_name = "TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256",
     .iana_value = { TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256 },
     .key_exchange_alg = &s2n_dhe,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -683,6 +717,7 @@ struct s2n_cipher_suite s2n_dhe_rsa_with_chacha20_poly1305_sha256 = /* 0xCC,0xAA
 struct s2n_cipher_suite s2n_ecdhe_kyber_rsa_with_aes_256_gcm_sha384 = /* 0xFF, 0x0C */ {
     .available = 0,
     .name = "ECDHE-KYBER-RSA-AES256-GCM-SHA384",
+    .iana_name = "TLS_ECDHE_KYBER_RSA_WITH_AES_256_GCM_SHA384",
     .iana_value = { TLS_ECDHE_KYBER_RSA_WITH_AES_256_GCM_SHA384 },
     .key_exchange_alg = &s2n_hybrid_ecdhe_kem,
     .auth_method = S2N_AUTHENTICATION_RSA,
@@ -697,8 +732,9 @@ struct s2n_cipher_suite s2n_ecdhe_kyber_rsa_with_aes_256_gcm_sha384 = /* 0xFF, 0
 struct s2n_cipher_suite s2n_tls13_aes_128_gcm_sha256 = {
     .available = 0,
     .name = "TLS_AES_128_GCM_SHA256",
+    .iana_name = "TLS_AES_128_GCM_SHA256",
     .iana_value = { TLS_AES_128_GCM_SHA256 },
-    .key_exchange_alg = NULL,
+    .key_exchange_alg = &s2n_tls13_kex,
     .auth_method = S2N_AUTHENTICATION_METHOD_TLS13,
     .record_alg = NULL,
     .all_record_algs = { &s2n_tls13_record_alg_aes128_gcm },
@@ -711,8 +747,9 @@ struct s2n_cipher_suite s2n_tls13_aes_128_gcm_sha256 = {
 struct s2n_cipher_suite s2n_tls13_aes_256_gcm_sha384 = {
     .available = 0,
     .name = "TLS_AES_256_GCM_SHA384",
+    .iana_name = "TLS_AES_256_GCM_SHA384",
     .iana_value = { TLS_AES_256_GCM_SHA384 },
-    .key_exchange_alg = NULL,
+    .key_exchange_alg = &s2n_tls13_kex,
     .auth_method = S2N_AUTHENTICATION_METHOD_TLS13,
     .record_alg = NULL,
     .all_record_algs = { &s2n_tls13_record_alg_aes256_gcm },
@@ -725,8 +762,9 @@ struct s2n_cipher_suite s2n_tls13_aes_256_gcm_sha384 = {
 struct s2n_cipher_suite s2n_tls13_chacha20_poly1305_sha256 = {
     .available = 0,
     .name = "TLS_CHACHA20_POLY1305_SHA256",
+    .iana_name = "TLS_CHACHA20_POLY1305_SHA256",
     .iana_value = { TLS_CHACHA20_POLY1305_SHA256 },
-    .key_exchange_alg = NULL,
+    .key_exchange_alg = &s2n_tls13_kex,
     .auth_method = S2N_AUTHENTICATION_METHOD_TLS13,
     .record_alg = NULL,
     .all_record_algs = { &s2n_tls13_record_alg_chacha20_poly1305 },
@@ -837,17 +875,18 @@ const struct s2n_cipher_preferences cipher_preferences_test_all_tls12 = {
  * in order of IANA value. Exposed for the "test_all_fips" cipher preference list.
  */
 static struct s2n_cipher_suite *s2n_all_fips_cipher_suites[] = {
-    &s2n_rsa_with_3des_ede_cbc_sha,           /* 0x00,0x0A */
-    &s2n_rsa_with_aes_128_cbc_sha,            /* 0x00,0x2F */
-    &s2n_rsa_with_aes_256_cbc_sha,            /* 0x00,0x35 */
-    &s2n_rsa_with_aes_128_cbc_sha256,         /* 0x00,0x3C */
-    &s2n_rsa_with_aes_256_cbc_sha256,         /* 0x00,0x3D */
+    &s2n_dhe_rsa_with_aes_128_cbc_sha,        /* 0x00,0x33 */
+    &s2n_dhe_rsa_with_aes_256_cbc_sha,        /* 0x00,0x39 */
     &s2n_dhe_rsa_with_aes_128_cbc_sha256,     /* 0x00,0x67 */
     &s2n_dhe_rsa_with_aes_256_cbc_sha256,     /* 0x00,0x6B */
-    &s2n_rsa_with_aes_128_gcm_sha256,         /* 0x00,0x9C */
-    &s2n_rsa_with_aes_256_gcm_sha384,         /* 0x00,0x9D */
     &s2n_dhe_rsa_with_aes_128_gcm_sha256,     /* 0x00,0x9E */
     &s2n_dhe_rsa_with_aes_256_gcm_sha384,     /* 0x00,0x9F */
+    &s2n_tls13_aes_128_gcm_sha256,            /* 0x13,0x01 */
+    &s2n_tls13_aes_256_gcm_sha384,            /* 0x13,0x02 */
+    &s2n_ecdhe_ecdsa_with_aes_128_cbc_sha,    /* 0xC0,0x09 */
+    &s2n_ecdhe_ecdsa_with_aes_256_cbc_sha,    /* 0xC0,0x0A */
+    &s2n_ecdhe_rsa_with_aes_128_cbc_sha,      /* 0xC0,0x13 */
+    &s2n_ecdhe_rsa_with_aes_256_cbc_sha,      /* 0xC0,0x14 */
     &s2n_ecdhe_ecdsa_with_aes_128_cbc_sha256, /* 0xC0,0x23 */
     &s2n_ecdhe_ecdsa_with_aes_256_cbc_sha384, /* 0xC0,0x24 */
     &s2n_ecdhe_rsa_with_aes_128_cbc_sha256,   /* 0xC0,0x27 */
@@ -998,7 +1037,8 @@ int s2n_cipher_suites_init(void)
 
         /* Initialize SSLv3 cipher suite if SSLv3 utilizes a different record algorithm */
         if (cur_suite->sslv3_record_alg && cur_suite->sslv3_record_alg->cipher->is_available()) {
-            struct s2n_blob cur_suite_mem = { .data = (uint8_t *) cur_suite, .size = sizeof(struct s2n_cipher_suite) };
+            struct s2n_blob cur_suite_mem = { 0 };
+            POSIX_GUARD(s2n_blob_init(&cur_suite_mem, (uint8_t *) cur_suite, sizeof(struct s2n_cipher_suite)));
             struct s2n_blob new_suite_mem = { 0 };
             POSIX_GUARD(s2n_dup(&cur_suite_mem, &new_suite_mem));
 
@@ -1050,7 +1090,6 @@ S2N_RESULT s2n_cipher_suites_cleanup(void)
         * cleanup in later versions */
 #endif
     }
-
     return S2N_RESULT_OK;
 }
 
@@ -1088,7 +1127,7 @@ int s2n_set_cipher_as_client(struct s2n_connection *conn, uint8_t wire[S2N_TLS_C
     POSIX_ENSURE_REF(conn->secure);
     POSIX_ENSURE_REF(conn->secure->cipher_suite);
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
     POSIX_ENSURE_REF(security_policy);
 
@@ -1156,7 +1195,7 @@ int s2n_set_cipher_as_client(struct s2n_connection *conn, uint8_t wire[S2N_TLS_C
 
 static int s2n_wire_ciphers_contain(const uint8_t *match, const uint8_t *wire, uint32_t count, uint32_t cipher_suite_len)
 {
-    for (uint32_t i = 0; i < count; i++) {
+    for (size_t i = 0; i < count; i++) {
         const uint8_t *theirs = wire + (i * cipher_suite_len) + (cipher_suite_len - S2N_TLS_CIPHER_SUITE_LEN);
 
         if (!memcmp(match, theirs, S2N_TLS_CIPHER_SUITE_LEN)) {
@@ -1172,7 +1211,7 @@ bool s2n_cipher_suite_uses_chacha20_alg(struct s2n_cipher_suite *cipher_suite)
     return cipher_suite && cipher_suite->record_alg && cipher_suite->record_alg->cipher == &s2n_chacha20_poly1305;
 }
 
-/* Iff the server has enabled allow_chacha20_boosting and the client has a chacha20 cipher suite as its most 
+/* Iff the server has enabled allow_chacha20_boosting and the client has a chacha20 cipher suite as its most
  * preferred cipher suite, then we have mutual chacha20 boosting support.
  */
 static S2N_RESULT s2n_validate_chacha20_boosting(const struct s2n_cipher_preferences *cipher_preferences, const uint8_t *wire,
@@ -1209,7 +1248,6 @@ static int s2n_set_cipher_as_server(struct s2n_connection *conn, uint8_t *wire,
     if (conn->client_protocol_version < conn->server_protocol_version) {
         uint8_t fallback_scsv[S2N_TLS_CIPHER_SUITE_LEN] = { TLS_FALLBACK_SCSV };
         if (s2n_wire_ciphers_contain(fallback_scsv, wire, count, cipher_suite_len)) {
-            conn->closed = 1;
             POSIX_BAIL(S2N_ERR_FALLBACK_DETECTED);
         }
     }
@@ -1232,7 +1270,7 @@ static int s2n_set_cipher_as_server(struct s2n_connection *conn, uint8_t *wire,
         conn->secure_renegotiation = 1;
     }
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     const struct s2n_cipher_preferences *cipher_preferences = security_policy->cipher_preferences;
@@ -1248,7 +1286,7 @@ static int s2n_set_cipher_as_server(struct s2n_connection *conn, uint8_t *wire,
      * other cipher suites.
      *
      * If no mutually supported cipher suites are found, we choose one with a version
-     * too high for the current connection (higher_vers_match). 
+     * too high for the current connection (higher_vers_match).
      */
     for (size_t i = 0; i < cipher_preferences->count; i++) {
         const uint8_t *ours = cipher_preferences->suites[i]->iana_value;
@@ -1277,19 +1315,15 @@ static int s2n_set_cipher_as_server(struct s2n_connection *conn, uint8_t *wire,
                 continue;
             }
 
-            /* TLS 1.3 does not include key exchange in cipher suites */
-            if (match->minimum_required_tls_version < S2N_TLS13) {
-                /* If the kex is not supported continue to the next candidate */
-                bool kex_supported = false;
-                POSIX_GUARD_RESULT(s2n_kex_supported(match, conn, &kex_supported));
-                if (!kex_supported) {
-                    continue;
-                }
-
-                /* If the kex is not configured correctly continue to the next candidate */
-                if (s2n_result_is_error(s2n_configure_kex(match, conn))) {
-                    continue;
-                }
+            /* If the kex is not supported continue to the next candidate */
+            bool kex_supported = false;
+            POSIX_GUARD_RESULT(s2n_kex_supported(match, conn, &kex_supported));
+            if (!kex_supported) {
+                continue;
+            }
+            /* If the kex is not configured correctly continue to the next candidate */
+            if (s2n_result_is_error(s2n_configure_kex(match, conn))) {
+                continue;
             }
 
             /**
