@@ -18,14 +18,14 @@
 #include "crypto/s2n_hash.h"
 
 /* Codes from http://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-5 */
-#define TLS_NULL_WITH_NULL_NULL             0x00, 0x00
-#define TLS_RSA_WITH_AES_256_CBC_SHA256     0x00, 0x3D
-#define TLS_RSA_WITH_AES_256_CBC_SHA        0x00, 0x35
-#define TLS_RSA_WITH_AES_128_CBC_SHA256     0x00, 0x3C
-#define TLS_RSA_WITH_AES_128_CBC_SHA        0x00, 0x2F
-#define TLS_RSA_WITH_3DES_EDE_CBC_SHA       0x00, 0x0A
-#define TLS_RSA_WITH_RC4_128_MD5            0x00, 0x04
-#define TLS_RSA_WITH_RC4_128_SHA            0x00, 0x05
+#define TLS_NULL_WITH_NULL_NULL         0x00, 0x00
+#define TLS_RSA_WITH_AES_256_CBC_SHA256 0x00, 0x3D
+#define TLS_RSA_WITH_AES_256_CBC_SHA    0x00, 0x35
+#define TLS_RSA_WITH_AES_128_CBC_SHA256 0x00, 0x3C
+#define TLS_RSA_WITH_AES_128_CBC_SHA    0x00, 0x2F
+#define TLS_RSA_WITH_3DES_EDE_CBC_SHA   0x00, 0x0A
+#define TLS_RSA_WITH_RC4_128_MD5        0x00, 0x04
+#define TLS_RSA_WITH_RC4_128_SHA        0x00, 0x05
 
 #define TLS_DHE_RSA_WITH_AES_128_CBC_SHA    0x00, 0x33
 #define TLS_DHE_RSA_WITH_AES_128_CBC_SHA256 0x00, 0x67
@@ -33,58 +33,63 @@
 #define TLS_DHE_RSA_WITH_AES_256_CBC_SHA256 0x00, 0x6B
 #define TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA   0x00, 0x16
 
-#define TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA       0xC0, 0x09
-#define TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256    0xC0, 0x23
-#define TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA       0xC0, 0x0A
-#define TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384    0xC0, 0x24
+#define TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA    0xC0, 0x09
+#define TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256 0xC0, 0x23
+#define TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA    0xC0, 0x0A
+#define TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384 0xC0, 0x24
 
-#define TLS_ECDHE_RSA_WITH_RC4_128_SHA           0xC0, 0x11
-#define TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA       0xC0, 0x13
-#define TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256    0xC0, 0x27
-#define TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA       0xC0, 0x14
-#define TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384    0xC0, 0x28
-#define TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA      0xC0, 0x12
+#define TLS_ECDHE_RSA_WITH_RC4_128_SHA        0xC0, 0x11
+#define TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA    0xC0, 0x13
+#define TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256 0xC0, 0x27
+#define TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA    0xC0, 0x14
+#define TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384 0xC0, 0x28
+#define TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA   0xC0, 0x12
 
-#define TLS_RSA_WITH_AES_128_GCM_SHA256          0x00, 0x9C
-#define TLS_RSA_WITH_AES_256_GCM_SHA384          0x00, 0x9D
-#define TLS_DHE_RSA_WITH_AES_128_GCM_SHA256      0x00, 0x9E
-#define TLS_DHE_RSA_WITH_AES_256_GCM_SHA384      0x00, 0x9F
-#define TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256  0xC0, 0x2B
-#define TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384  0xC0, 0x2C
-#define TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256    0xC0, 0x2F
-#define TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384    0xC0, 0x30
+#define TLS_RSA_WITH_AES_128_GCM_SHA256         0x00, 0x9C
+#define TLS_RSA_WITH_AES_256_GCM_SHA384         0x00, 0x9D
+#define TLS_DHE_RSA_WITH_AES_128_GCM_SHA256     0x00, 0x9E
+#define TLS_DHE_RSA_WITH_AES_256_GCM_SHA384     0x00, 0x9F
+#define TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256 0xC0, 0x2B
+#define TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384 0xC0, 0x2C
+#define TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256   0xC0, 0x2F
+#define TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384   0xC0, 0x30
 
-#define TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256    0xCC, 0xA8
-#define TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256  0xCC, 0xA9
-#define TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256      0xCC, 0xAA
+#define TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256   0xCC, 0xA8
+#define TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256 0xCC, 0xA9
+#define TLS_DHE_RSA_WITH_CHACHA20_POLY1305_SHA256     0xCC, 0xAA
 
 /* TLS 1.2 hybrid post-quantum definitions from https://tools.ietf.org/html/draft-campagna-tls-bike-sike-hybrid */
 #define TLS_ECDHE_KYBER_RSA_WITH_AES_256_GCM_SHA384 0xFF, 0x0C
-#define TLS_EXTENSION_PQ_KEM_PARAMETERS 0xFE01
-#define TLS_PQ_KEM_EXTENSION_ID_KYBER_512_R3 28
+#define TLS_EXTENSION_PQ_KEM_PARAMETERS             0xFE01
+#define TLS_PQ_KEM_EXTENSION_ID_KYBER_512_R3        28
 
 /* TLS 1.3 hybrid post-quantum definitions are from the proposed reserved range defined
  * in https://tools.ietf.org/html/draft-stebila-tls-hybrid-design. Values for interoperability are defined in
- * https://github.com/open-quantum-safe/openssl/blob/OQS-OpenSSL_1_1_1-stable/oqs-template/oqs-kem-info.md */
+ * https://github.com/open-quantum-safe/oqs-provider/blob/main/oqs-template/oqs-kem-info.md and
+ * https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml
+ */
 #define TLS_PQ_KEM_GROUP_ID_X25519_KYBER_512_R3     0x2F39
 #define TLS_PQ_KEM_GROUP_ID_SECP256R1_KYBER_512_R3  0x2F3A
-
+#define TLS_PQ_KEM_GROUP_ID_SECP384R1_KYBER_768_R3  0x2F3C
+#define TLS_PQ_KEM_GROUP_ID_SECP521R1_KYBER_1024_R3 0x2F3D
+#define TLS_PQ_KEM_GROUP_ID_X25519_KYBER_768_R3     0x6399
+#define TLS_PQ_KEM_GROUP_ID_SECP256R1_KYBER_768_R3  0x639A
 
 /* From https://tools.ietf.org/html/rfc7507 */
-#define TLS_FALLBACK_SCSV                   0x56, 0x00
-#define TLS_EMPTY_RENEGOTIATION_INFO_SCSV   0x00, 0xff
+#define TLS_FALLBACK_SCSV                 0x56, 0x00
+#define TLS_EMPTY_RENEGOTIATION_INFO_SCSV 0x00, 0xff
 
 /* TLS 1.3 cipher suites from https://tools.ietf.org/html/rfc8446#appendix-B.4 */
-#define TLS_AES_128_GCM_SHA256              0x13, 0x01
-#define TLS_AES_256_GCM_SHA384              0x13, 0x02
-#define TLS_CHACHA20_POLY1305_SHA256        0x13, 0x03
-#define TLS_AES_128_CCM_SHA256              0x13, 0x04
-#define TLS_AES_128_CCM_8_SHA256            0x13, 0x05
+#define TLS_AES_128_GCM_SHA256       0x13, 0x01
+#define TLS_AES_256_GCM_SHA384       0x13, 0x02
+#define TLS_CHACHA20_POLY1305_SHA256 0x13, 0x03
+#define TLS_AES_128_CCM_SHA256       0x13, 0x04
+#define TLS_AES_128_CCM_8_SHA256     0x13, 0x05
 
 /* TLS extensions from https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml */
-#define TLS_EXTENSION_SERVER_NAME           0
-#define TLS_EXTENSION_MAX_FRAG_LEN          1
-#define TLS_EXTENSION_STATUS_REQUEST        5
+#define TLS_EXTENSION_SERVER_NAME          0
+#define TLS_EXTENSION_MAX_FRAG_LEN         1
+#define TLS_EXTENSION_STATUS_REQUEST       5
 #define TLS_EXTENSION_SUPPORTED_GROUPS     10
 #define TLS_EXTENSION_EC_POINT_FORMATS     11
 #define TLS_EXTENSION_SIGNATURE_ALGORITHMS 13
@@ -100,7 +105,7 @@
  * iana value. However, Openssl does have a chosen value for this extension 
  * and that is what is used here.
 */
-#define TLS_EXTENSION_NPN                  13172
+#define TLS_EXTENSION_NPN 13172
 
 /* TLS 1.3 extensions from https://tools.ietf.org/html/rfc8446#section-4.2 */
 #define TLS_EXTENSION_EARLY_DATA             42
@@ -119,45 +124,40 @@
  *#      quic_transport_parameters(0x39), (65535)
  *#   } ExtensionType;
  */
-#define TLS_QUIC_TRANSPORT_PARAMETERS      0x39
+#define TLS_EXTENSION_QUIC_TRANSPORT_PARAMETERS 0x39
 
 /* TLS SignatureScheme (Backwards compatible with SigHash and SigAlg values above) */
 /* Defined here: https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-signaturescheme */
-#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA1             0x0201
-#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA224           0x0301
-#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA256           0x0401
-#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA384           0x0501
-#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA512           0x0601
+#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA1   0x0201
+#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA224 0x0301
+#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA256 0x0401
+#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA384 0x0501
+#define TLS_SIGNATURE_SCHEME_RSA_PKCS1_SHA512 0x0601
 
 /* In TLS 1.0 and 1.1 the hard-coded default scheme was RSA_PKCS1_MD5_SHA1, but there's no IANA defined backwards
  * compatible value for that Scheme for TLS 1.2 and 1.3. So we define an internal value in the private range that won't
  * match anything in the valid range so that all TLS Versions can use the same SignatureScheme negotiation abstraction
  * layer. This scheme isn't in any preference list, so it can't be negotiated even if a client sent it in its pref list. */
-#define TLS_SIGNATURE_SCHEME_PRIVATE_INTERNAL_RSA_PKCS1_MD5_SHA1         0xFFFF
+#define TLS_SIGNATURE_SCHEME_PRIVATE_INTERNAL_RSA_PKCS1_MD5_SHA1 0xFFFF
 
 /* TLS 1.2 Backwards Compatible ECDSA Schemes */
-#define TLS_SIGNATURE_SCHEME_ECDSA_SHA1                 0x0203
-#define TLS_SIGNATURE_SCHEME_ECDSA_SHA224               0x0303
-#define TLS_SIGNATURE_SCHEME_ECDSA_SHA256               0x0403
-#define TLS_SIGNATURE_SCHEME_ECDSA_SHA384               0x0503
-#define TLS_SIGNATURE_SCHEME_ECDSA_SHA512               0x0603
+#define TLS_SIGNATURE_SCHEME_ECDSA_SHA1   0x0203
+#define TLS_SIGNATURE_SCHEME_ECDSA_SHA224 0x0303
+#define TLS_SIGNATURE_SCHEME_ECDSA_SHA256 0x0403
+#define TLS_SIGNATURE_SCHEME_ECDSA_SHA384 0x0503
+#define TLS_SIGNATURE_SCHEME_ECDSA_SHA512 0x0603
 
-/* TLS 1.3 ECDSA Signature Schemes */
-#define TLS_SIGNATURE_SCHEME_ECDSA_SECP256R1_SHA256     0x0403
-#define TLS_SIGNATURE_SCHEME_ECDSA_SECP384R1_SHA384     0x0503
-#define TLS_SIGNATURE_SCHEME_ECDSA_SECP521R1_SHA512     0x0603
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA256        0x0804
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA384        0x0805
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA512        0x0806
-#define TLS_SIGNATURE_SCHEME_ED25519                    0x0807
-#define TLS_SIGNATURE_SCHEME_ED448                      0x0808
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA256         0x0809
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA384         0x080A
-#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA512         0x080B
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA256 0x0804
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA384 0x0805
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_RSAE_SHA512 0x0806
+#define TLS_SIGNATURE_SCHEME_ED25519             0x0807
+#define TLS_SIGNATURE_SCHEME_ED448               0x0808
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA256  0x0809
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA384  0x080A
+#define TLS_SIGNATURE_SCHEME_RSA_PSS_PSS_SHA512  0x080B
 
-
-#define TLS_SIGNATURE_SCHEME_LEN                        2
-#define TLS_SIGNATURE_SCHEME_LIST_MAX_LEN               64
+#define TLS_SIGNATURE_SCHEME_LEN          2
+#define TLS_SIGNATURE_SCHEME_LIST_MAX_LEN 128
 
 /* The TLS record types we support */
 #define SSLv2_CLIENT_HELLO     1
@@ -169,16 +169,16 @@
 /* Elliptic curve formats from http://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-9
  * Only uncompressed is supported.
  */
-#define TLS_EC_FORMAT_UNCOMPRESSED               0
-#define TLS_EC_FORMAT_ANSIX962_COMPRESSED_PRIME  1
-#define TLS_EC_FORMAT_ANSIX962_COMPRESSED_CHAR2  2
+#define TLS_EC_FORMAT_UNCOMPRESSED              0
+#define TLS_EC_FORMAT_ANSIX962_COMPRESSED_PRIME 1
+#define TLS_EC_FORMAT_ANSIX962_COMPRESSED_CHAR2 2
 
 /* Elliptic curves from https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-8 */
-#define TLS_EC_CURVE_SECP_256_R1           23
-#define TLS_EC_CURVE_SECP_384_R1           24
-#define TLS_EC_CURVE_SECP_521_R1           25
-#define TLS_EC_CURVE_ECDH_X25519           29
-#define TLS_EC_CURVE_ECDH_X448             30
+#define TLS_EC_CURVE_SECP_256_R1 23
+#define TLS_EC_CURVE_SECP_384_R1 24
+#define TLS_EC_CURVE_SECP_521_R1 25
+#define TLS_EC_CURVE_ECDH_X25519 29
+#define TLS_EC_CURVE_ECDH_X448   30
 
 /* Ethernet maximum transmission unit (MTU)
  * MTU is usually associated with the Ethernet protocol,
@@ -189,10 +189,10 @@
 #define IP_V4_HEADER_LENGTH 20
 #define IP_V6_HEADER_LENGTH 40
 
-#define TCP_HEADER_LENGTH 20
+#define TCP_HEADER_LENGTH  20
 #define TCP_OPTIONS_LENGTH 40
 
-#define S2N_TLS_MAX_FRAG_LEN_EXT_NONE   0
+#define S2N_TLS_MAX_FRAG_LEN_EXT_NONE 0
 
 /* The maximum size of an SSL2 message is 2^14 - 1, as neither of the first two
  * bits in the length field are usable. Per;
@@ -209,19 +209,19 @@
  * length that is designed to maximize throughput (fewer MACs per byte transferred
  * and better efficiency of crypto engines).
  */
-#define S2N_SMALL_RECORD_LENGTH (1500 - 20 - 20 - 20)
+#define S2N_SMALL_RECORD_LENGTH   (1500 - 20 - 20 - 20)
 #define S2N_SMALL_FRAGMENT_LENGTH (S2N_SMALL_RECORD_LENGTH - S2N_TLS_RECORD_HEADER_LENGTH)
 
 /* Testing in the wild has found 8k max record sizes give a good balance of low latency
  * and throughput.
  */
-#define S2N_DEFAULT_RECORD_LENGTH 8092
+#define S2N_DEFAULT_RECORD_LENGTH   8092
 #define S2N_DEFAULT_FRAGMENT_LENGTH (S2N_DEFAULT_RECORD_LENGTH - S2N_TLS_RECORD_HEADER_LENGTH)
 
 /* S2N_LARGE_RECORD_LENGTH is used for initializing output buffers, we use the largest
  * possible value of all supported protocols to avoid branching at runtime
  */
-#define S2N_LARGE_RECORD_LENGTH S2N_TLS_MAXIMUM_RECORD_LENGTH
+#define S2N_LARGE_RECORD_LENGTH   S2N_TLS_MAXIMUM_RECORD_LENGTH
 #define S2N_LARGE_FRAGMENT_LENGTH S2N_TLS_MAXIMUM_FRAGMENT_LENGTH
 
 /* Cap dynamic record resize threshold to 8M */
@@ -237,6 +237,6 @@
 #define S2N_ALERT_LENGTH 2
 
 /* Handshake messages have their own header too */
-#define TLS_HANDSHAKE_HEADER_LENGTH   4
+#define TLS_HANDSHAKE_HEADER_LENGTH 4
 
 #define S2N_MAX_SERVER_NAME 255
