@@ -57,6 +57,7 @@ static const char *AWS_PYOBJECT_KEY_MAX_RECONNECT_DELAY_MS = "max_reconnect_dela
 static const char *AWS_PYOBJECT_KEY_MIN_CONNECTED_TIME_TO_RESET_RECONNECT_DELAY_MS =
     "min_connected_time_to_reset_reconnect_delay_ms";
 static const char *AWS_PYOBJECT_KEY_PING_TIMEOUT_MS = "ping_timeout_ms";
+static const char *AWS_PYOBJECT_KEY_CONNACK_TIMEOUT_MS = "connack_timeout_ms";
 static const char *AWS_PYOBJECT_KEY_ACK_TIMEOUT_SECONDS = "ack_timeout_seconds";
 
 #define KEEP_ALIVE_INTERVAL_SECONDS 1200
@@ -260,7 +261,7 @@ static void s_on_publish_received(const struct aws_mqtt5_packet_publish_view *pu
     result = PyObject_CallMethod(
         client->client_core,
         "_on_publish",
-        "(y#iOs#OiOIOHs#z#Os#O)",
+        "(y#iOs#OiOIOHs#y#Os#O)",
         /* y */ publish_packet->payload.ptr,
         /* # */ publish_packet->payload.len,
         /* i */ (int)publish_packet->qos,
@@ -276,7 +277,7 @@ static void s_on_publish_received(const struct aws_mqtt5_packet_publish_view *pu
         /* H */ (unsigned short)(publish_packet->topic_alias ? *publish_packet->topic_alias : 0),
         /* s */ publish_packet->response_topic ? publish_packet->response_topic->ptr : NULL,
         /* # */ publish_packet->response_topic ? publish_packet->response_topic->len : 0,
-        /* z */ publish_packet->correlation_data ? publish_packet->correlation_data->ptr : NULL,
+        /* y */ publish_packet->correlation_data ? publish_packet->correlation_data->ptr : NULL,
         /* # */ publish_packet->correlation_data ? publish_packet->correlation_data->len : 0,
         /* O */ subscription_identifier_count > 0 ? subscription_identifier_list : Py_None,
         /* s */ publish_packet->content_type ? publish_packet->content_type->ptr : NULL,
@@ -363,7 +364,7 @@ static void s_lifecycle_event_connection_success(
     result = PyObject_CallMethod(
         client->client_core,
         "_on_lifecycle_connection_success",
-        "(OiOIOHOiOOOIs#OIs#OOOOOOOOHs#s#iIHIHHHOOOOO)",
+        "(OiOIOHOiOOOIs#OIs#OOOOOOOOHs#s#iIHIHHHOOOOOs#)",
         /* connack packet  */
         /* O */ connack->session_present ? Py_True : Py_False,
         /* i */ (int)connack->reason_code,
@@ -412,7 +413,9 @@ static void s_lifecycle_event_connection_success(
         /* O */ settings->wildcard_subscriptions_available ? Py_True : Py_False,
         /* O */ settings->subscription_identifiers_available ? Py_True : Py_False,
         /* O */ settings->shared_subscriptions_available ? Py_True : Py_False,
-        /* O */ settings->rejoined_session ? Py_True : Py_False);
+        /* O */ settings->rejoined_session ? Py_True : Py_False,
+        /* s */ settings->client_id_storage.len > 0 ? settings->client_id_storage.buffer : NULL,
+        /* # */ settings->client_id_storage.len);
     if (!result) {
         PyErr_WriteUnraisable(PyErr_Occurred());
     }
@@ -709,13 +712,13 @@ PyObject *aws_py_mqtt5_ws_handshake_transform_complete(PyObject *self, PyObject
 
     PyObject *exception_py;
     PyObject *ws_transform_capsule;
-    if (!PyArg_ParseTuple(args, "OO", &exception_py, &ws_transform_capsule)) {
+    int error_code = AWS_ERROR_SUCCESS;
+    if (!PyArg_ParseTuple(args, "OOi", &exception_py, &ws_transform_capsule, &error_code)) {
         return NULL;
     }
 
-    int error_code = AWS_ERROR_SUCCESS;
-    if (exception_py != Py_None) {
-        /* TODO: Translate Python exception to aws error. In the meantime here's a catch-all. */
+    if (exception_py != Py_None && error_code == AWS_ERROR_SUCCESS) {
+        /* Fallback code for if the error source was outside the CRT native implementation */
         error_code = AWS_ERROR_HTTP_CALLBACK_FAILURE;
     }
 
@@ -809,11 +812,11 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
     PyObject *socket_options_py;
     PyObject *tls_ctx_py;
     PyObject *proxy_options_py;
-    uint16_t port;
+    uint32_t port;
 
     /* Connect Options */
     struct aws_byte_cursor client_id;          /* optional */
-    PyObject *keep_alive_interval_sec_py;      /* uint16_t */
+    PyObject *keep_alive_interval_sec_py;      /* optional uint16_t */
     struct aws_byte_cursor username;           /* optional */
     struct aws_byte_cursor password;           /* optional */
     PyObject *session_expiry_interval_sec_py;  /* optional uint32_t */
@@ -846,6 +849,7 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
     PyObject *max_reconnect_delay_ms_py;                         /* optional uint64_t */
     PyObject *min_connected_time_to_reset_reconnect_delay_ms_py; /* optional uint64_t */
     PyObject *ping_timeout_ms_py;                                /* optional uint32_t */
+    PyObject *connack_timeout_ms_py;                             /* optional uint32_t */
     PyObject *ack_timeout_seconds_py;                            /* optional uint32_t */
     PyObject *topic_aliasing_options_py;                         /* optional TopicAliasingOptions */
     /* Callbacks */
@@ -854,11 +858,11 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
 
     if (!PyArg_ParseTuple(
             args,
-            "Os#HOOOOz#Oz#z#OOOOOOOOOz*Oz#OOOz#z*z#OOOOOOOOOOOOO",
+            "Os#IOOOOz#Oz#z#OOOOOOOOOz*Oz#OOOz#z*z#OOOOOOOOOOOOOO",
             /* O */ &self_py,
             /* s */ &host_name.ptr,
             /* # */ &host_name.len,
-            /* H */ &port,
+            /* I */ &port,
             /* O */ &bootstrap_py,
             /* O */ &socket_options_py,
             /* O */ &tls_ctx_py,
@@ -904,6 +908,7 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
             /* O */ &max_reconnect_delay_ms_py,
             /* O */ &min_connected_time_to_reset_reconnect_delay_ms_py,
             /* O */ &ping_timeout_ms_py,
+            /* O */ &connack_timeout_ms_py,
             /* O */ &ack_timeout_seconds_py,
             /* O */ &topic_aliasing_options_py,
 
@@ -1077,6 +1082,18 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
         goto done;
     }
 
+    uint32_t connack_timeout_ms_tmp = 0;
+    if (PyObject_GetAsOptionalUint32(
+            connack_timeout_ms_py,
+            AWS_PYOBJECT_KEY_CLIENT_OPTIONS,
+            AWS_PYOBJECT_KEY_CONNACK_TIMEOUT_MS,
+            &connack_timeout_ms_tmp)) {
+        client_options.connack_timeout_ms = connack_timeout_ms_tmp;
+    }
+    if (PyErr_Occurred()) {
+        goto done;
+    }
+
     uint32_t ack_timeout_seconds_tmp = 0;
     if (PyObject_GetAsOptionalUint32(
             ack_timeout_seconds_py,
