@@ -615,13 +615,13 @@ PyObject *aws_py_mqtt_ws_handshake_transform_complete(PyObject *self, PyObject *
 
     PyObject *exception_py;
     PyObject *ws_transform_capsule;
-    if (!PyArg_ParseTuple(args, "OO", &exception_py, &ws_transform_capsule)) {
+    int error_code = AWS_ERROR_SUCCESS;
+    if (!PyArg_ParseTuple(args, "OOi", &exception_py, &ws_transform_capsule, &error_code)) {
         return NULL;
     }
 
-    int error_code = AWS_ERROR_SUCCESS;
-    if (exception_py != Py_None) {
-        /* TODO: Translate Python exception to aws error. In the meantime here's a catch-all. */
+    if (exception_py != Py_None && error_code == AWS_ERROR_SUCCESS) {
+        /* Fallback code for if the error source was outside the CRT native implementation */
         error_code = AWS_ERROR_HTTP_CALLBACK_FAILURE;
     }
 
@@ -645,7 +645,7 @@ PyObject *aws_py_mqtt_client_connection_connect(PyObject *self, PyObject *args)
     Py_ssize_t client_id_len;
     const char *server_name;
     Py_ssize_t server_name_len;
-    uint16_t port_number;
+    uint32_t port_number;
     PyObject *socket_options_py;
     PyObject *tls_ctx_py;
     uint64_t reconnect_min_timeout_secs;
@@ -663,7 +663,7 @@ PyObject *aws_py_mqtt_client_connection_connect(PyObject *self, PyObject *args)
     PyObject *proxy_options_py;
     if (!PyArg_ParseTuple(
             args,
-            "Os#s#HOOKKHIIOz#z#OOO",
+            "Os#s#IOOKKHIIOz#z#OOO",
             &impl_capsule,
             &client_id,
             &client_id_len,
