@@ -35,9 +35,11 @@ class MqttConnectionTest(NativeResourceTest):
             tls_context,
             port=8883,
             use_static_singletons=False,
+            client_id=None,
             on_connection_success_callback=None,
             on_connection_failure_callback=None,
-            on_connection_closed_callback=None):
+            on_connection_closed_callback=None,
+            on_connection_resumed_callback=None):
         if use_static_singletons:
             client = Client(tls_ctx=tls_context)
         else:
@@ -48,12 +50,13 @@ class MqttConnectionTest(NativeResourceTest):
 
         connection = Connection(
             client=client,
-            client_id=create_client_id(),
+            client_id=client_id if client_id else create_client_id(),
             host_name=endpoint,
             port=port,
             on_connection_closed=on_connection_closed_callback,
             on_connection_failure=on_connection_failure_callback,
-            on_connection_success=on_connection_success_callback)
+            on_connection_success=on_connection_success_callback,
+            on_connection_resumed=on_connection_resumed_callback)
         return connection
 
     def test_connect_disconnect(self):
@@ -165,8 +168,8 @@ class MqttConnectionTest(NativeResourceTest):
             host_name=test_input_endpoint,
             port=8883,
             will=Will(self.TEST_TOPIC, QoS.AT_LEAST_ONCE, self.TEST_MSG, False),
-            ping_timeout_ms=500,
-            keep_alive_secs=1
+            ping_timeout_ms=10000,
+            keep_alive_secs=30
         )
         connection.connect().result(TIMEOUT)
 
@@ -174,7 +177,9 @@ class MqttConnectionTest(NativeResourceTest):
             client=client,
             client_id=create_client_id(),
             host_name=test_input_endpoint,
-            port=8883
+            port=8883,
+            ping_timeout_ms=10000,
+            keep_alive_secs=30
         )
         subscriber.connect().result(TIMEOUT)
 
@@ -190,16 +195,18 @@ class MqttConnectionTest(NativeResourceTest):
         self.assertEqual(self.TEST_TOPIC, suback['topic'])
         self.assertIs(QoS.AT_LEAST_ONCE, suback['qos'])
 
-        # Disconnect the will client to send the will
-        time.sleep(1.1)  # wait 1.1 seconds to ensure we can make another client ID connect
+        # wait a few seconds to ensure we can make another client ID connect (don't trigger IoT Core limit)
+        time.sleep(2)
+
+        # Disconnect the will client to send the will by making another connection with the same client id
         disconnecter = Connection(
             client=client,
             client_id=will_client_id,
             host_name=test_input_endpoint,
             port=8883,
             will=Will(self.TEST_TOPIC, QoS.AT_LEAST_ONCE, self.TEST_MSG, False),
-            ping_timeout_ms=500,
-            keep_alive_secs=1
+            ping_timeout_ms=10000,
+            keep_alive_secs=30
         )
         disconnecter.connect().result(TIMEOUT)
 
@@ -400,18 +407,18 @@ class MqttConnectionTest(NativeResourceTest):
         test_tls_opts = TlsContextOptions.create_client_with_mtls_from_path(test_input_cert, test_input_key)
         test_tls = ClientTlsContext(test_tls_opts)
 
-        onConnectionSuccessFuture = Future()
-        onConnectionClosedFuture = Future()
+        on_connection_success_future = Future()
+        on_connection_closed_future = Future()
 
         def on_connection_success_callback(connection, callback_data: OnConnectionSuccessData):
-            onConnectionSuccessFuture.set_result(
+            on_connection_success_future.set_result(
                 {'return_code': callback_data.return_code, "session_present": callback_data.session_present})
 
         def on_connection_failure_callback(connection, callback_data: OnConnectionFailureData):
             pass
 
         def on_connection_closed_callback(connection, callback_data: OnConnectionClosedData):
-            onConnectionClosedFuture.set_result({})
+            on_connection_closed_future.set_result({})
 
         connection = self._create_connection(
             endpoint=test_input_endpoint,
@@ -420,11 +427,11 @@ class MqttConnectionTest(NativeResourceTest):
             on_connection_failure_callback=on_connection_failure_callback,
             on_connection_closed_callback=on_connection_closed_callback)
         connection.connect().result(TIMEOUT)
-        successData = onConnectionSuccessFuture.result(TIMEOUT)
-        self.assertEqual(successData['return_code'], ConnectReturnCode.ACCEPTED)
-        self.assertEqual(successData['session_present'], False)
+        success_data = on_connection_success_future.result(TIMEOUT)
+        self.assertEqual(success_data['return_code'], ConnectReturnCode.ACCEPTED)
+        self.assertEqual(success_data['session_present'], False)
         connection.disconnect().result(TIMEOUT)
-        onConnectionClosedFuture.result(TIMEOUT)
+        on_connection_closed_future.result(TIMEOUT)
 
     def test_connect_disconnect_with_callbacks_unhappy(self):
         test_input_endpoint = _get_env_variable("AWS_TEST_MQTT311_IOT_CORE_HOST")
@@ -433,13 +440,13 @@ class MqttConnectionTest(NativeResourceTest):
         test_tls_opts = TlsContextOptions.create_client_with_mtls_from_path(test_input_cert, test_input_key)
         test_tls = ClientTlsContext(test_tls_opts)
 
-        onConnectionFailureFuture = Future()
+        on_onnection_failure_future = Future()
 
         def on_connection_success_callback(connection, callback_data: OnConnectionSuccessData):
             pass
 
         def on_connection_failure_callback(connection, callback_data: OnConnectionFailureData):
-            onConnectionFailureFuture.set_result({'error': callback_data.error})
+            on_onnection_failure_future.set_result({'error': callback_data.error})
 
         def on_connection_closed_callback(connection, callback_data: OnConnectionClosedData):
             pass
@@ -459,8 +466,77 @@ class MqttConnectionTest(NativeResourceTest):
             exception_occurred = True
         self.assertTrue(exception_occurred, "Exception did not occur when connecting with invalid arguments!")
 
-        failureData = onConnectionFailureFuture.result(TIMEOUT)
-        self.assertTrue(failureData['error'] is not None)
+        failure_data = on_onnection_failure_future.result(TIMEOUT)
+        self.assertTrue(failure_data['error'] is not None)
+
+    def test_connect_disconnect_with_callbacks_happy_on_resume(self):
+        # Check that an on_connection_success callback fires on a resumed connection.
+
+        # NOTE Since there is no mocked server available on this abstraction level, the only sensible approach
+        # is to interrupt a connection, and wait for it to be resumed automatically. For that, another client
+        # with the same client_id connects to the server and then immediately disconnects.
+
+        test_input_endpoint = _get_env_variable("AWS_TEST_MQTT311_IOT_CORE_HOST")
+        test_input_cert = _get_env_variable("AWS_TEST_MQTT311_IOT_CORE_RSA_CERT")
+        test_input_key = _get_env_variable("AWS_TEST_MQTT311_IOT_CORE_RSA_KEY")
+        test_tls_opts = TlsContextOptions.create_client_with_mtls_from_path(test_input_cert, test_input_key)
+        test_tls = ClientTlsContext(test_tls_opts)
+
+        on_connection_success_future = Future()
+        on_connection_closed_future = Future()
+        on_connection_resumed_future = Future()
+
+        def on_connection_success_callback(connection, callback_data: OnConnectionSuccessData):
+            on_connection_success_future.set_result(
+                {'return_code': callback_data.return_code, "session_present": callback_data.session_present})
+
+        def on_connection_closed_callback(connection, callback_data: OnConnectionClosedData):
+            on_connection_closed_future.set_result({})
+
+        def on_connection_resumed_callback(connection, return_code: ConnectReturnCode, session_present):
+            on_connection_resumed_future.set_result(
+                {'return_code': return_code, "session_present": session_present})
+
+        connection = self._create_connection(
+            endpoint=test_input_endpoint,
+            tls_context=test_tls,
+            on_connection_success_callback=on_connection_success_callback,
+            on_connection_closed_callback=on_connection_closed_callback,
+            on_connection_resumed_callback=on_connection_resumed_callback)
+        connection.connect().result(TIMEOUT)
+        success_data = on_connection_success_future.result(TIMEOUT)
+        self.assertEqual(success_data['return_code'], ConnectReturnCode.ACCEPTED)
+        self.assertEqual(success_data['session_present'], False)
+
+        # Reset the future for the reconnect attempt.
+        on_connection_success_future = Future()
+
+        on_connection_success_future_dup = Future()
+
+        def on_connection_success_callback_dup(connection, callback_data: OnConnectionSuccessData):
+            on_connection_success_future_dup.set_result({})
+
+        # Reuse the same client_id to displace the first client.
+        connection_dup = self._create_connection(
+            endpoint=test_input_endpoint,
+            tls_context=test_tls,
+            client_id=connection.client_id,
+            on_connection_success_callback=on_connection_success_callback_dup)
+
+        connection_dup.connect().result(TIMEOUT)
+        on_connection_success_future_dup.result(TIMEOUT)
+        connection_dup.disconnect().result(TIMEOUT)
+
+        # After the second client disconnects, the first one should reconnect,
+        # and on_connection_success callback should be fired once again.
+        on_connection_resumed_future.result(TIMEOUT)
+        success_data = on_connection_success_future.result(TIMEOUT)
+
+        self.assertEqual(success_data['return_code'], ConnectReturnCode.ACCEPTED)
+        self.assertEqual(success_data['session_present'], False)
+
+        connection.disconnect().result(TIMEOUT)
+        on_connection_closed_future.result(TIMEOUT)
 
     # ==============================================================
     #             MOSQUITTO CONNECTION TESTS
