@@ -4,6 +4,7 @@
 from concurrent.futures import Future
 from awscrt import mqtt5, io, http, exceptions
 from test import NativeResourceTest
+from threading import Lock
 import os
 import unittest
 import uuid
@@ -904,6 +905,7 @@ class Mqtt5ClientTest(NativeResourceTest):
         self.assertTrue(callbacks.negotiated_settings.subscription_identifiers_available)
         self.assertTrue(callbacks.negotiated_settings.shared_subscriptions_available)
         self.assertFalse(callbacks.negotiated_settings.rejoined_session)
+        self.assertEqual(callbacks.negotiated_settings.client_id, client_id)
 
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
@@ -1007,6 +1009,167 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    sub1_callbacks = False
+    sub2_callbacks = False
+    total_callbacks = 0
+    all_packets_received = Future()
+    mutex = Lock()
+    received_subscriptions = [0] * 10
+
+    def subscriber1_callback(self, publish_received_data: mqtt5.PublishReceivedData):
+        self.mutex.acquire()
+        var = publish_received_data.publish_packet.payload
+        self.received_subscriptions[int(var)] = 1
+        self.sub1_callbacks = True
+        self.total_callbacks = self.total_callbacks + 1
+        if self.total_callbacks == 10:
+            self.all_packets_received.set_result(None)
+        self.mutex.release()
+
+    def subscriber2_callback(self, publish_received_data: mqtt5.PublishReceivedData):
+        self.mutex.acquire()
+        var = publish_received_data.publish_packet.payload
+        self.received_subscriptions[int(var)] = 1
+        self.sub2_callbacks = True
+        self.total_callbacks = self.total_callbacks + 1
+        if self.total_callbacks == 10:
+            self.all_packets_received.set_result(None)
+        self.mutex.release()
+
+    def test_operation_shared_subscription(self):
+        input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
+        input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
+        input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
+
+        client_id_subscriber1 = create_client_id()
+        client_id_subscriber2 = create_client_id()
+        client_id_publisher = create_client_id()
+
+        testTopic = "test/MQTT5_Binding_Python_" + client_id_publisher
+        sharedTopicfilter = "$share/crttest/test/MQTT5_Binding_Python_" + client_id_publisher
+
+        tls_ctx_options = io.TlsContextOptions.create_client_with_mtls_from_path(
+            input_cert,
+            input_key
+        )
+
+        # subscriber 1
+        connect_subscriber1_options = mqtt5.ConnectPacket(client_id=client_id_subscriber1)
+        subscriber1_generic_callback = Mqtt5TestCallbacks()
+        subscriber1_options = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883,
+            tls_ctx=io.ClientTlsContext(tls_ctx_options),
+            connect_options=connect_subscriber1_options,
+            on_publish_callback_fn=self.subscriber1_callback,
+            on_lifecycle_event_stopped_fn=subscriber1_generic_callback.on_lifecycle_stopped,
+            on_lifecycle_event_attempting_connect_fn=subscriber1_generic_callback.on_lifecycle_attempting_connect,
+            on_lifecycle_event_connection_success_fn=subscriber1_generic_callback.on_lifecycle_connection_success,
+            on_lifecycle_event_connection_failure_fn=subscriber1_generic_callback.on_lifecycle_connection_failure
+        )
+        subscriber1_client = mqtt5.Client(client_options=subscriber1_options)
+
+        # subscriber 2
+        connect_subscriber2_options = mqtt5.ConnectPacket(client_id=client_id_subscriber2)
+        subscriber2_generic_callback = Mqtt5TestCallbacks()
+        subscriber2_options = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883,
+            tls_ctx=io.ClientTlsContext(tls_ctx_options),
+            connect_options=connect_subscriber2_options,
+            on_publish_callback_fn=self.subscriber2_callback,
+            on_lifecycle_event_stopped_fn=subscriber2_generic_callback.on_lifecycle_stopped,
+            on_lifecycle_event_attempting_connect_fn=subscriber2_generic_callback.on_lifecycle_attempting_connect,
+            on_lifecycle_event_connection_success_fn=subscriber2_generic_callback.on_lifecycle_connection_success,
+            on_lifecycle_event_connection_failure_fn=subscriber2_generic_callback.on_lifecycle_connection_failure
+        )
+        subscriber2_client = mqtt5.Client(client_options=subscriber2_options)
+
+        # publisher
+        connect_publisher_options = mqtt5.ConnectPacket(client_id=client_id_publisher)
+        publisher_generic_callback = Mqtt5TestCallbacks()
+
+        publisher_options = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883,
+            tls_ctx=io.ClientTlsContext(tls_ctx_options),
+            connect_options=connect_publisher_options,
+            on_lifecycle_event_stopped_fn=publisher_generic_callback.on_lifecycle_stopped,
+            on_lifecycle_event_attempting_connect_fn=publisher_generic_callback.on_lifecycle_attempting_connect,
+            on_lifecycle_event_connection_success_fn=publisher_generic_callback.on_lifecycle_connection_success,
+            on_lifecycle_event_connection_failure_fn=publisher_generic_callback.on_lifecycle_connection_failure
+        )
+        publisher_client = mqtt5.Client(client_options=publisher_options)
+
+        print("Connecting all 3 clients\n")
+        subscriber1_client.start()
+        subscriber1_generic_callback.future_connection_success.result(TIMEOUT)
+
+        subscriber2_client.start()
+        subscriber2_generic_callback.future_connection_success.result(TIMEOUT)
+
+        publisher_client.start()
+        publisher_generic_callback.future_connection_success.result(TIMEOUT)
+        print("All clients connected\n")
+
+        # Subscriber 1
+        subscriptions = []
+        subscriptions.append(mqtt5.Subscription(topic_filter=sharedTopicfilter, qos=mqtt5.QoS.AT_LEAST_ONCE))
+        subscribe_packet = mqtt5.SubscribePacket(
+            subscriptions=subscriptions)
+        subscribe_future = subscriber1_client.subscribe(subscribe_packet=subscribe_packet)
+        suback_packet1 = subscribe_future.result(TIMEOUT)
+        self.assertIsInstance(suback_packet1, mqtt5.SubackPacket)
+
+        # Subscriber 2
+        subscriptions2 = []
+        subscriptions2.append(mqtt5.Subscription(topic_filter=sharedTopicfilter, qos=mqtt5.QoS.AT_LEAST_ONCE))
+        subscribe_packet2 = mqtt5.SubscribePacket(
+            subscriptions=subscriptions2)
+        subscribe_future2 = subscriber2_client.subscribe(subscribe_packet=subscribe_packet2)
+        suback_packet2 = subscribe_future2.result(TIMEOUT)
+        self.assertIsInstance(suback_packet2, mqtt5.SubackPacket)
+
+        publishes = 10
+        for x in range(0, publishes):
+            packet = mqtt5.PublishPacket(
+                payload=f"{x}",
+                qos=mqtt5.QoS.AT_LEAST_ONCE,
+                topic=testTopic
+            )
+            publish_future = publisher_client.publish(packet)
+            publish_future.result(TIMEOUT)
+
+        self.all_packets_received.result(TIMEOUT)
+
+        topic_filters = []
+        topic_filters.append(testTopic)
+        unsubscribe_packet = mqtt5.UnsubscribePacket(topic_filters=testTopic)
+
+        unsubscribe_future = subscriber1_client.unsubscribe(unsubscribe_packet)
+        unsuback_packet = unsubscribe_future.result(TIMEOUT)
+        self.assertIsInstance(unsuback_packet, mqtt5.UnsubackPacket)
+
+        unsubscribe_future = subscriber2_client.unsubscribe(unsubscribe_packet)
+        unsuback_packet = unsubscribe_future.result(TIMEOUT)
+        self.assertIsInstance(unsuback_packet, mqtt5.UnsubackPacket)
+
+        self.assertEqual(self.sub1_callbacks, True)
+        self.assertEqual(self.sub2_callbacks, True)
+        self.assertEqual(self.total_callbacks, 10)
+
+        for e in self.received_subscriptions:
+            self.assertEqual(e, 1)
+
+        subscriber1_client.stop()
+        subscriber1_generic_callback.future_stopped.result(TIMEOUT)
+
+        subscriber2_client.stop()
+        subscriber2_generic_callback.future_stopped.result(TIMEOUT)
+
+        publisher_client.stop()
+        publisher_generic_callback.future_stopped.result(TIMEOUT)
+
     def test_operation_will(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
@@ -1067,6 +1230,98 @@ class Mqtt5ClientTest(NativeResourceTest):
         client2.stop()
         callbacks2.future_stopped.result(TIMEOUT)
 
+    def do_will_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
+                                      expected_correlation_data_bytes, expected_correlation_data):
+        input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
+        input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
+        input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
+
+        client_id_publisher = create_client_id()
+        topic_filter = "test/MQTT5_Binding_Python_" + client_id_publisher
+
+        payload = "TEST WILL"
+        payload_bytes = payload.encode("utf-8")
+
+        will_packet = mqtt5.PublishPacket(
+            payload="TEST WILL",
+            qos=mqtt5.QoS.AT_LEAST_ONCE,
+            topic=topic_filter,
+            correlation_data_bytes=outbound_correlation_data_bytes,
+            correlation_data=outbound_correlation_data
+        )
+
+        tls_ctx_options = io.TlsContextOptions.create_client_with_mtls_from_path(
+            input_cert,
+            input_key
+        )
+
+        client_options1 = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883
+        )
+        client_options1.connect_options = mqtt5.ConnectPacket(client_id=client_id_publisher,
+                                                              will_delay_interval_sec=0,
+                                                              will=will_packet)
+        client_options1.tls_ctx = io.ClientTlsContext(tls_ctx_options)
+        callbacks1 = Mqtt5TestCallbacks()
+        client1 = self._create_client(client_options=client_options1, callbacks=callbacks1)
+        client1.start()
+        callbacks1.future_connection_success.result(TIMEOUT)
+
+        client_options2 = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883
+        )
+        client_options2.connect_options = mqtt5.ConnectPacket(client_id=create_client_id())
+        client_options2.tls_ctx = io.ClientTlsContext(tls_ctx_options)
+        callbacks2 = Mqtt5TestCallbacks()
+        client2 = self._create_client(client_options=client_options2, callbacks=callbacks2)
+        client2.start()
+        callbacks2.future_connection_success.result(TIMEOUT)
+
+        subscriptions = []
+        subscriptions.append(mqtt5.Subscription(topic_filter=topic_filter, qos=mqtt5.QoS.AT_LEAST_ONCE))
+        subscribe_packet = mqtt5.SubscribePacket(
+            subscriptions=subscriptions)
+        subscribe_future = client2.subscribe(subscribe_packet=subscribe_packet)
+        suback_packet = subscribe_future.result(TIMEOUT)
+        self.assertIsInstance(suback_packet, mqtt5.SubackPacket)
+
+        disconnect_packet = mqtt5.DisconnectPacket(reason_code=mqtt5.DisconnectReasonCode.DISCONNECT_WITH_WILL_MESSAGE)
+        client1.stop(disconnect_packet=disconnect_packet)
+        callbacks1.future_stopped.result(TIMEOUT)
+
+        received_will = callbacks2.future_publish_received.result(TIMEOUT)
+        self.assertIsInstance(received_will, mqtt5.PublishPacket)
+        self.assertEqual(received_will.payload, payload_bytes)
+        self.assertEqual(received_will.correlation_data_bytes, expected_correlation_data_bytes)
+        self.assertEqual(received_will.correlation_data, expected_correlation_data)
+
+        client2.stop()
+        callbacks2.future_stopped.result(TIMEOUT)
+
+    def test_will_correlation_data_bytes_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_will_correlation_data_test(correlation_data, None, correlation_data, None)
+
+    def test_will_correlation_data_bytes_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self.do_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
+
+    def test_will_correlation_data_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_correlation_data_test(None, correlation_data, correlation_data, None)
+
+    def test_will_correlation_data_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self.do_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+
+    def test_will_correlation_data_bytes_binary_precedence(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
+
     def test_operation_binary_publish(self):
         input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
         input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
@@ -1129,6 +1384,81 @@ class Mqtt5ClientTest(NativeResourceTest):
         client.stop()
         callbacks.future_stopped.result(TIMEOUT)
 
+    def do_correlation_data_test(self, outbound_correlation_data_bytes, outbound_correlation_data,
+                                 expected_correlation_data_bytes, expected_correlation_data):
+        input_host_name = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_HOST")
+        input_cert = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_CERT")
+        input_key = _get_env_variable("AWS_TEST_MQTT5_IOT_CORE_RSA_KEY")
+
+        client_id = create_client_id()
+        topic_filter = "test/MQTT5_Binding_Python_" + client_id
+        payload = bytearray(os.urandom(256))
+
+        client_options = mqtt5.ClientOptions(
+            host_name=input_host_name,
+            port=8883
+        )
+        tls_ctx_options = io.TlsContextOptions.create_client_with_mtls_from_path(
+            input_cert,
+            input_key
+        )
+        client_options.tls_ctx = io.ClientTlsContext(tls_ctx_options)
+        callbacks = Mqtt5TestCallbacks()
+        client = self._create_client(client_options=client_options, callbacks=callbacks)
+        client.start()
+        callbacks.future_connection_success.result(TIMEOUT)
+
+        subscriptions = []
+        subscriptions.append(mqtt5.Subscription(topic_filter=topic_filter, qos=mqtt5.QoS.AT_LEAST_ONCE))
+        subscribe_packet = mqtt5.SubscribePacket(
+            subscriptions=subscriptions)
+        subscribe_future = client.subscribe(subscribe_packet=subscribe_packet)
+        suback_packet = subscribe_future.result(TIMEOUT)
+        self.assertIsInstance(suback_packet, mqtt5.SubackPacket)
+
+        publish_packet = mqtt5.PublishPacket(
+            payload=payload,
+            topic=topic_filter,
+            qos=mqtt5.QoS.AT_LEAST_ONCE,
+            correlation_data_bytes=outbound_correlation_data_bytes,
+            correlation_data=outbound_correlation_data)
+
+        publish_future = client.publish(publish_packet=publish_packet)
+        publish_completion_data = publish_future.result(TIMEOUT)
+        puback_packet = publish_completion_data.puback
+        self.assertIsInstance(puback_packet, mqtt5.PubackPacket)
+
+        received_publish = callbacks.future_publish_received.result(TIMEOUT)
+        self.assertIsInstance(received_publish, mqtt5.PublishPacket)
+        self.assertEqual(received_publish.payload, payload)
+        self.assertEqual(received_publish.correlation_data_bytes, expected_correlation_data_bytes)
+        self.assertEqual(received_publish.correlation_data, expected_correlation_data)
+
+        client.stop()
+        callbacks.future_stopped.result(TIMEOUT)
+
+    def test_operation_publish_correlation_data_bytes_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_correlation_data_test(correlation_data, None, correlation_data, None)
+
+    def test_operation_publish_correlation_data_bytes_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self.do_correlation_data_test(correlation_data, None, correlation_data_as_bytes, correlation_data)
+
+    def test_operation_publish_correlation_data_binary(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_correlation_data_test(None, correlation_data, correlation_data, None)
+
+    def test_operation_publish_correlation_data_string(self):
+        correlation_data = "CorrelationData"
+        correlation_data_as_bytes = correlation_data.encode('utf-8')
+        self.do_correlation_data_test(None, correlation_data, correlation_data_as_bytes, correlation_data)
+
+    def test_operation_publish_correlation_data_bytes_binary_precedence(self):
+        correlation_data = bytearray(os.urandom(64))
+        self.do_correlation_data_test(correlation_data, "Ignored", correlation_data, None)
+
     # ==============================================================
     #             OPERATION ERROR TEST CASES
     # ==============================================================
